@@ -35,11 +35,18 @@ public class IJ {
 	public static Font font10 = new Font("SansSerif", Font.PLAIN, 10);
 	/** SansSerif, plain, 12-point font */
 	public static Font font12 = ImageJ.SansSerif12;
+	/** SansSerif, plain, 14-point font */
+	public static Font font14 = ImageJ.SansSerif14;
 	
 	/** Image display modes */
 	public static final int COMPOSITE=1, COLOR=2, GRAYSCALE=3;
 	
+	/** @deprecated */
 	public static final String URL = "http://imagej.nih.gov/ij";
+	
+	/** ImageJ website */
+	public static final String URL2 = "http://imagej.net/ij";
+
 	public static final int ALL_KEYS = -1;
 	
 	/** Use setDebugMode(boolean) to enable/disable debug mode. */
@@ -62,7 +69,6 @@ public class IJ {
 	private static boolean macroRunning;
 	private static Thread previousThread;
 	private static TextPanel logPanel;
-	private static boolean checkForDuplicatePlugins = true;		
 	private static ClassLoader classLoader;
 	private static boolean memMessageDisplayed;
 	private static long maxMemory;
@@ -211,10 +217,12 @@ public class IJ {
  			else
 				new PlugInFilterRunner(thePlugIn, commandName, arg);
 		} catch (ClassNotFoundException e) {
-			log("Plugin or class not found: \"" + className + "\"\n(" + e+")");
-			String path = Prefs.getCustomPropsPath();
-			if (path!=null);
-				log("Error may be due to custom properties at " + path);
+			if (!(className!=null && className.startsWith("ij.plugin.MacAdapter"))) {
+				log("Plugin or class not found: \"" + className + "\"\n(" + e+")");
+				String path = Prefs.getCustomPropsPath();
+				if (path!=null);
+					log("Error may be due to custom properties at " + path);
+			}
 		}
 		catch (InstantiationException e) {log("Unable to load plugin (ins)");}
 		catch (IllegalAccessException e) {log("Unable to load plugin, possibly \nbecause it is not public.");}
@@ -226,11 +234,6 @@ public class IJ {
 		if (IJ.debugMode)
 			IJ.log("runUserPlugIn: "+className+", arg="+argument(arg));
 		if (applet!=null) return null;
-		if (checkForDuplicatePlugins) {
-			// check for duplicate classes and jars in the plugins folder
-			IJ.runPlugIn("ij.plugin.ClassChecker", "");
-			checkForDuplicatePlugins = false;
-		}
 		if (createNewLoader)
 			classLoader = null;
 		ClassLoader loader = getClassLoader();
@@ -384,8 +387,8 @@ public class IJ {
 			commandTable.put("Line Graph (21K)", "Line Graph");			
 			commandTable.put("Mitosis (26MB, 5D stack)", "Mitosis (5D stack)");
 			commandTable.put("MRI Stack (528K)", "MRI Stack");
-			commandTable.put("M51 Galaxy (177K, 16-bits)", "M51 Galaxy (16-bits))");
-			commandTable.put("Neuron (1.6M, 5 channels", "Neuron (5 channels");
+			commandTable.put("M51 Galaxy (177K, 16-bits)", "M51 Galaxy (16-bits)");
+			commandTable.put("Neuron (1.6M, 5 channels)", "Neuron (5 channels)");
 			commandTable.put("Nile Bend (1.9M)", "Nile Bend");			
 			commandTable.put("Organ of Corti (2.8M, 4D stack)", "Organ of Corti (4D stack)");
 			commandTable.put("Particles (75K)", "Particles");
@@ -466,6 +469,67 @@ public class IJ {
 		}
 	}
 	
+	/**Displays a message in the status bar and flashes
+	 * either the status bar or the active image.<br>
+	 * See: http://wsr.imagej.net/macros/FlashingStatusMessages.txt
+	*/ 
+	public static void showStatus(String message, String options) {
+		showStatus(message);
+		if (options==null)
+			return;
+		options = options.replace("flash", "");
+		options = options.replace("ms", "");
+		Color optionalColor = null;
+		int index1 = options.indexOf("#");
+		if (index1>=0) {  // hex color?
+			int index2 = options.indexOf(" ", index1);
+			if (index2==-1) index2 = options.length();
+			String hexColor = options.substring(index1, index2);
+			optionalColor = Colors.decode(hexColor, null);
+			options = options.replace(hexColor, "");
+		}
+		if (optionalColor==null) {  // "red", "green", etc.
+			for (String c : Colors.colors) {
+				if (options.contains(c)) {
+					optionalColor = Colors.getColor(c, ImageJ.backgroundColor);
+					options = options.replace(c, "");
+					break;
+				}
+			}
+		}
+		boolean flashImage = options.contains("image");
+		Color defaultColor = new Color(255,255,245);
+		int defaultDelay = 500;
+		ImagePlus imp = WindowManager.getCurrentImage();
+		if (flashImage) {
+			options = options.replace("image", "");
+			if (imp!=null && imp.getWindow()!=null) {
+				defaultColor = Color.black;
+				defaultDelay = 100;
+			}
+			else
+				flashImage = false;
+		}
+		Color color = optionalColor!=null?optionalColor:defaultColor;
+		int delay = (int)Tools.parseDouble(options, defaultDelay);
+		if (delay>8000)
+			delay = 8000;
+		String colorString = null;
+		ImageJ ij = IJ.getInstance();
+		if (flashImage) {
+			Color previousColor = imp.getWindow().getBackground();
+			imp.getWindow().setBackground(color);
+			if (delay>0) {
+				wait(delay);			
+				imp.getWindow().setBackground(previousColor);
+			}
+		} else if (ij!=null) {
+			ij.getStatusBar().setBackground(color);
+			wait(delay);
+			ij.getStatusBar().setBackground(ij.backgroundColor);
+		}
+	}
+	
 	/**
 	* @deprecated
 	* replaced by IJ.log(), ResultsTable.setResult() and TextWindow.append().
@@ -712,10 +776,12 @@ public class IJ {
 	}
 
 	/**	Updates the progress bar, where the length of the bar is set to
-    (<code>currentValue+1)/finalValue</code> of the maximum bar length.
-    The bar is erased if <code>currentValue&gt;=finalValue</code>. 
-    The bar is updated only if more than 90 ms have passed since the last call.
-    Does nothing if the ImageJ window is not present. */
+     * (<code>currentValue+1)/finalValue</code> of the maximum bar length.
+     * The bar is erased if <code>currentValue&gt;=finalValue</code>. 
+     * The bar is updated only if more than 90 ms have passed since the
+     * last call. Displays subordinate progress bars as dots if
+     * 'currentIndex' is negative (example: Plugins/Utilities/Benchmark).
+    */
     public static void showProgress(int currentIndex, int finalIndex) {
 		if (progressBar!=null) {
 			progressBar.show(currentIndex, finalIndex);
@@ -998,9 +1064,7 @@ public class IJ {
 		return str;
 	}
 
-	/** Adds the specified class to a Vector to keep it from being garbage
-	collected, which would cause the classes static fields to be reset. 
-	Probably not needed with Java 1.2 or later. */
+	/** Obsolete */
 	public static void register(Class c) {
 		if (ij!=null) ij.register(c);
 	}
@@ -1025,6 +1089,11 @@ public class IJ {
 		return shiftDown;
 	}
 	
+	/* Sets the specified key "down", where 'key'
+	 * is KeyEvent.VK_CONTROL, KeyEvent.VK_META, 
+	 * KeyEvent.VK_ALT, KeyEvent.VK_SHIFT,
+	 * KeyEvent.VK_SPACE or KeyEvent.VK_ESCAPE.
+	*/
 	public static void setKeyDown(int key) {
 		switch (key) {
 			case KeyEvent.VK_CONTROL:
@@ -1054,6 +1123,11 @@ public class IJ {
 		}
 	}
 
+	/* Sets the specified key "up", where 'key'
+	 * is KeyEvent.VK_CONTROL, KeyEvent.VK_META, 
+	 * KeyEvent.VK_ALT, KeyEvent.VK_SHIFT,
+	 * KeyEvent.VK_SPACE or IJ.ALL_KEYS (-1).
+	*/
 	public static void setKeyUp(int key) {
 		switch (key) {
 			case KeyEvent.VK_CONTROL: controlDown=false; break;
@@ -1383,8 +1457,17 @@ public class IJ {
 		setRawThreshold(img, lowerThreshold, upperThreshold, displayMode);
 	}
 
+	/** This is a version of setThreshold() that uses raw (uncalibrated)
+	 * values in the range 0-255 for 8-bit images and 0-65535 for 16-bit
+	 * images and the "Red" LUT display mode.
+	*/
+	public static void setRawThreshold(ImagePlus img, double lowerThreshold, double upperThreshold) {
+		setRawThreshold(img, lowerThreshold, upperThreshold, null);
+	}
+
 	/** This is a version of setThreshold() that always uses raw (uncalibrated) values
-		 in the range 0-255 for 8-bit images and 0-65535 for 16-bit images. */
+	 * in the range 0-255 for 8-bit images and 0-65535 for 16-bit images.
+	*/
 	public static void setRawThreshold(ImagePlus img, double lowerThreshold, double upperThreshold, String displayMode) {
 		int mode = ImageProcessor.RED_LUT;
 		if (displayMode!=null) {
@@ -1404,6 +1487,7 @@ public class IJ {
 		}
 	}
 
+	/** Replaced by ImagePlus.setAutoThreshold(). */
 	public static void setAutoThreshold(ImagePlus imp, String method) {
 		ImageProcessor ip = imp.getProcessor();
 		if (ip instanceof ColorProcessor)
@@ -1424,7 +1508,10 @@ public class IJ {
 	}
 	
 	private static void setStackThreshold(ImagePlus imp, ImageProcessor ip, String method) {
-		boolean darkBackground = method.indexOf("dark")!=-1;
+		boolean darkBackground = method.contains("dark");
+		boolean histo16 = imp.getBitDepth()==16 && method.contains("16");
+		//IJ.log("setStackThreshold: "+histo16+" "+method);
+		boolean addOne = !method.contains("16");
 		int measurements = Analyzer.getMeasurements();
 		Analyzer.setMeasurements(Measurements.AREA+Measurements.MIN_MAX);
 		ImageStatistics stats = new StackStatistics(imp);
@@ -1435,26 +1522,34 @@ public class IJ {
 			min = stats.min;
 			max = stats.max;
 		}
-		int threshold = thresholder.getThreshold(method, stats.histogram);
+		int[] histogram = stats.histogram;
+		if (histo16)
+			histogram = stats.histogram16;
+		int threshold = thresholder.getThreshold(method, histogram);
 		double lower, upper;
+		double tmax = 255.0;
+		if (histogram.length>256)
+			tmax = 65535.0;
 		if (darkBackground) {
 			if (ip.isInvertedLut())
 				{lower=0.0; upper=threshold;}
 			else
-				{lower=threshold+1; upper=255.0;}
+				{lower=threshold+(addOne?1:0); upper=tmax;}
 		} else {
 			if (ip.isInvertedLut())
-				{lower=threshold+1; upper=255.0;}
+				{lower=threshold+(addOne?1:0); upper=tmax;}
 			else
 				{lower=0.0; upper=threshold;}
 		}
-		if (lower>255) lower = 255;
-		if (max>min) {
-			lower = min + (lower/255.0)*(max-min);
-			upper = min + (upper/255.0)*(max-min);
-		} else
-			lower = upper = min;
-		ip.setMinAndMax(min, max);
+		if (!histo16) {
+			if (lower>255) lower = 255;
+			if (max>min) {
+				lower = min + (lower/255.0)*(max-min);
+				upper = min + (upper/255.0)*(max-min);
+			} else
+				lower = upper = min;
+			ip.setMinAndMax(min, max);
+		}
 		ip.setThreshold(lower, upper, ImageProcessor.RED_LUT);
 		imp.updateAndDraw();
 	}
@@ -1495,6 +1590,7 @@ public class IJ {
 			ImageWindow win = imp.getWindow();
 			if (win!=null) {
 				win.toFront();
+				win.setState(Frame.NORMAL);
 				WindowManager.setWindow(win);
 			}
 			long start = System.currentTimeMillis();
@@ -1518,8 +1614,10 @@ public class IJ {
 
 	/** Activates the window with the specified title. */
 	public static void selectWindow(String title) {
-		if (title.equals("ImageJ")&&ij!=null)
-			{ij.toFront(); return;}
+		if (title.equals("ImageJ")&&ij!=null) {
+			ij.toFront();
+			return;
+		}
 		long start = System.currentTimeMillis();
 		while (System.currentTimeMillis()-start<3000) { // 3 sec timeout
 			Window win = WindowManager.getWindow(title);
@@ -1544,9 +1642,10 @@ public class IJ {
 	}
 	
 	static void selectWindow(Window win) {
-		if (win instanceof Frame)
+		if (win instanceof Frame) {
 			((Frame)win).toFront();
-		else
+			((Frame)win).setState(Frame.NORMAL);
+		} else
 			((Dialog)win).toFront();
 		long start = System.currentTimeMillis();
 		while (true) {
@@ -1571,9 +1670,7 @@ public class IJ {
 	}
 	
 	static void setColor(int red, int green, int blue, boolean foreground) {
-	    if (red<0) red=0; if (green<0) green=0; if (blue<0) blue=0; 
-	    if (red>255) red=255; if (green>255) green=255; if (blue>255) blue=255;  
-		Color c = new Color(red, green, blue);
+		Color c = Colors.toColor(red, green, blue);
 		if (foreground) {
 			Toolbar.setForegroundColor(c);
 			ImagePlus img = WindowManager.getCurrentImage();
@@ -1633,12 +1730,12 @@ public class IJ {
 				
 		}
 		Wand w = new Wand(ip);
-		double t1 = ip.getMinThreshold();
-		if (t1==ImageProcessor.NO_THRESHOLD || (ip.getLutUpdateMode()==ImageProcessor.NO_LUT_UPDATE&& tolerance>0.0)) {
+		if (!ip.isThreshold() || (ip.getLutUpdateMode()==ImageProcessor.NO_LUT_UPDATE&& tolerance>0.0)) {
 			w.autoOutline(x, y, tolerance, imode);
 			smooth = false;
-		} else
-			w.autoOutline(x, y, t1, ip.getMaxThreshold(), imode);
+		} else {
+			w.autoOutline(x, y, ip.getMinThreshold(), ip.getMaxThreshold(), imode);
+		}
 		if (w.npoints>0) {
 			Roi previousRoi = img.getRoi();
 			Roi roi = new PolygonRoi(w.xpoints, w.ypoints, w.npoints, Roi.TRACED_ROI);
@@ -1755,13 +1852,14 @@ public class IJ {
 	/** Returns the path to the specified directory if <code>title</code> is
 		"home" ("user.home"), "downloads", "startup",  "imagej" (ImageJ directory),
 		"plugins", "macros", "luts", "temp", "current", "default",
-		"image" (directory active image was loaded from) or "file" 
-		(directory most recently used to open or save a file),
-		otherwise displays a dialog and returns the path to the
-		directory selected by the user. Returns null if the specified
-		directory is not found or the user cancels the dialog box.
-		Also aborts the macro if the user cancels
-		the dialog box.*/
+		"image" (directory active image was loaded from), "file" 
+		(directory most recently used to open or save a file), "cwd"
+		(current working directory) or "preferences" (location of
+		"IJ_Prefs.txt" file), otherwise displays a dialog and
+		returns the path to the directory selected by the user. Returns
+		null if the specified directory is not found or the user cancels the
+		dialog box. Also aborts the macro if the user cancels the
+		dialog box.*/
 	public static String getDirectory(String title) {
 		String dir = null;
 		String title2 = title.toLowerCase(Locale.US);
@@ -1785,6 +1883,8 @@ public class IJ {
 			dir = Prefs.getImageJDir();
 		else if (title2.equals("current") || title2.equals("default"))
 			dir = OpenDialog.getDefaultDirectory();
+		else if (title2.equals("preferences"))
+			dir = Prefs.getPrefsDir();
 		else if (title2.equals("temp")) {
 			dir = System.getProperty("java.io.tmpdir");
 			if (isMacintosh()) dir = "/tmp/";
@@ -1795,11 +1895,15 @@ public class IJ {
 				dir = fi.directory;
 			} else
 				dir = null;
-		} else if (title2.equals("file")) {
+		} else if (title2.equals("file"))
 			dir = OpenDialog.getLastDirectory();
-		} else {
+		else if (title2.equals("cwd"))
+			dir = System.getProperty("user.dir");
+		else {
+			String defaultDir = OpenDialog.getDefaultDirectory();
 			DirectoryChooser dc = new DirectoryChooser(title);
 			dir = dc.getDirectory();
+			OpenDialog.setDefaultDirectory(defaultDir);
 			if (dir==null) Macro.abort();
 		}
 		dir = addSeparator(dir);
@@ -1865,12 +1969,13 @@ public class IJ {
 	}
 
 	/** Opens the specified file as a tiff, bmp, dicom, fits, pgm, gif, jpeg 
-		or text image and returns an ImagePlus object if successful.
-		Calls HandleExtraFileTypes plugin if the file type is not recognised.
-		Displays a file open dialog if 'path' is null or an empty string.
-		Note that 'path' can also be a URL. Some reader plugins, including
-		the Bio-Formats plugin, display the image and return null.
-		Use IJ.open() to display a file open dialog box.
+	 * or text image and returns an ImagePlus object if successful.
+	 * Calls HandleExtraFileTypes plugin if the file type is not recognised.
+	 * Displays a file open dialog if 'path' is null or an empty string.
+	 * Note that 'path' can also be a URL. Some reader plugins, including
+	 * the Bio-Formats plugin, display the image and return null.
+	 * Use IJ.open() to display a file open dialog box.
+	 * @see ij.io.Opener#openUsingBioFormats(String)
 	*/
 	public static ImagePlus openImage(String path) {
 		macroRunning = true;
@@ -1911,7 +2016,7 @@ public class IJ {
 			if (len>5242880L)
 				return "<Error: file is larger than 5MB>";
 			InputStream in = u.openStream();
-			BufferedReader br = new BufferedReader(new InputStreamReader(in));
+			BufferedReader br = new BufferedReader(new InputStreamReader(in,"UTF-8"));
 			sb = new StringBuffer() ;
 			String line;
 			while ((line=br.readLine()) != null)
@@ -1924,50 +2029,7 @@ public class IJ {
 			return new String(sb);
 		else
 			return "";
-	}
-	
-	/* 
-	public static void addRootCA() throws Exception {
-		String path = "/Users/wayne/Downloads/Certificates/lets-encrypt-x1-cross-signed.pem";
-		InputStream fis = new BufferedInputStream(new FileInputStream(path));
-		Certificate ca = CertificateFactory.getInstance("X.509").generateCertificate(fis);
-		KeyStore ks = KeyStore.getInstance(KeyStore.getDefaultType());
-		ks.load(null, null);
-		ks.setCertificateEntry(Integer.toString(1), ca);
-		TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
-		tmf.init(ks);
-		SSLContext ctx = SSLContext.getInstance("TLS");
-		ctx.init(null, tmf.getTrustManagers(), null); 
-		HttpsURLConnection.setDefaultSSLSocketFactory(ctx.getSocketFactory());
-	}
-	*/
-	
-	/*
-	// Create a new trust manager that trust all certificates
-	// http://stackoverflow.com/questions/10135074/download-file-from-https-server-using-java
-	private static void trustAllCerts() {
-		trustManagerCreated = true;
-		TrustManager[] trustAllCerts = new TrustManager[] {
-			new X509TrustManager() {
-				public java.security.cert.X509Certificate[] getAcceptedIssuers() {
-					return null;
-				}
-				public void checkClientTrusted (java.security.cert.X509Certificate[] certs, String authType) {
-				}
-				public void checkServerTrusted (java.security.cert.X509Certificate[] certs, String authType) {
-				}
-			}
-		};
-		// Activate the new trust manager
-		try {
-			SSLContext sc = SSLContext.getInstance("SSL");
-			sc.init(null, trustAllCerts, new java.security.SecureRandom());
-			HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
-		} catch (Exception e) {
-			IJ.log(""+e);
-		}
-	}
-	*/
+	}	
 
 	/** Saves the current image, lookup table, selection or text window to the specified file path. 
 		The path must end in ".tif", ".jpg", ".gif", ".zip", ".raw", ".avi", ".bmp", ".fits", ".pgm", ".png", ".lut", ".roi" or ".txt".  */
@@ -2383,7 +2445,7 @@ public class IJ {
 	
 	/** Returns, as an array of strings, a list of the LUTs in the 
 	 * Image/Lookup Tables menu.
-	 * @see ij.plugin#LutLoader.getLut
+	 * @see ij.plugin.LutLoader#getLut
 	 * See also: Help>Examples>JavaScript/Show all LUTs
 	 * and Image/Color/Display LUTs
 	*/
@@ -2395,10 +2457,8 @@ public class IJ {
 			return new String[0];
 		for (int i=0; i<lutsMenu.getItemCount(); i++) {
 			MenuItem menuItem = lutsMenu.getItem(i);
-			if (menuItem.getActionListeners().length == 0) // separator?
-				continue;
 			String label = menuItem.getLabel();
-			if (label.equals("Invert LUT") || label.equals("Apply LUT"))
+			if (label.equals("-") || label.equals("Invert LUT") || label.equals("Apply LUT"))
 				continue;
 			String command = (String)commands.get(label);
 			if (command==null || command.startsWith("ij.plugin.LutLoader"))
@@ -2433,10 +2493,14 @@ public class IJ {
 		PrintWriter pw = new PrintWriter(caw);
 		e.printStackTrace(pw);
 		String s = caw.toString();
+		String lineNumber = "";
 		if (s!=null && s.contains("ThreadDeath"))
 			return;
+		Interpreter interpreter = Thread.currentThread().getName().endsWith("Macro$") ? Interpreter.getInstance() : null;
+		if (interpreter!=null)
+			lineNumber = "\nMacro line number: " + interpreter.getLineNumber();
 		if (getInstance()!=null) {
-			s = IJ.getInstance().getInfo()+"\n \n"+s;
+			s = IJ.getInstance().getInfo()+lineNumber+"\n \n"+s;
 			new TextWindow("Exception", s, 500, 340);
 		} else
 			log(s);
