@@ -19,7 +19,7 @@ public class ImageStack {
 	private ColorModel cm;
 	private double min=Double.MAX_VALUE;
 	private double max;
-	private float[] cTable;
+	protected float[] cTable;
 	private int viewers;
 	private boolean signedInt;
 	
@@ -32,7 +32,7 @@ public class ImageStack {
 	}
 	
 	/** Creates a new, empty image stack with a capacity of 'size'.  All
-	     'size' slices and labels of this image stack are initially null.*/
+	     'size' slices and labels of this image stack are initially null. */
 	public ImageStack(int width, int height, int size) {
 		this.width = width;
 		this.height = height;
@@ -159,7 +159,7 @@ public class ImageStack {
 		label[n] = tempLabel;
 	}
 	
-	/** Deletes the specified slice, were 1<=n<=nslices. */
+	/** Deletes the specified slice, where {@literal 1<=n<=nslices}. */
 	public void deleteSlice(int n) {
 		if (n<1 || n>nSlices)
 			throw new IllegalArgumentException(outOfRange+n);
@@ -210,15 +210,14 @@ public class ImageStack {
 		}
 	}
 	
-	/** Returns the pixel array for the specified slice, were 1<=n<=nslices. */
+	/** Returns the pixel array for the specified slice, where {@literal 1<=n<=nslices}. */
 	public Object getPixels(int n) {
 		if (n<1 || n>nSlices)
 			throw new IllegalArgumentException(outOfRange+n);
 		return stack[n-1];
 	}
 	
-	/** Assigns a pixel array to the specified slice,
-		were 1<=n<=nslices. */
+	/** Assigns a pixel array to the specified slice, where {@literal 1<=n<=nslices}. */
 	public void setPixels(Object pixels, int n) {
 		if (n<1 || n>nSlices)
 			throw new IllegalArgumentException(outOfRange+n);
@@ -255,19 +254,31 @@ public class ImageStack {
 			return label;
 	}
 	
-	/** Returns the label of the specified slice, were 1<=n<=nslices.
-		Returns null if the slice does not have a label. For DICOM
-		and FITS stacks, labels may contain header information. */
+	/** Returns the label of the specified slice, where {@literal 1<=n<=nslices}.
+		Returns null if the slice does not have a label or 'n';
+		is out of range. For DICOM and FITS stacks, labels may
+		contain header information.
+	*/
 	public String getSliceLabel(int n) {
 		if (n<1 || n>nSlices)
-			throw new IllegalArgumentException(outOfRange+n);
-		return label[n-1];
+			return null;
+		else
+			return label[n-1];
 	}
 	
-	/** Returns a shortened version (up to the first 60 characters or first newline and 
-		suffix removed) of the label of the specified slice.
-		Returns null if the slice does not have a label. */
+	/** Returns a shortened version (up to the first 60 characters
+	 * or first newline), with the extension removed, of the specified
+	 * slice label, or null if the slice does not have a label.
+	*/
 	public String getShortSliceLabel(int n) {
+		return getShortSliceLabel(n, 60);
+	}
+
+	/** Returns a shortened version (up to the first 'max' characters
+	 * or first newline), with the extension removed, of the specified
+	 * slice label, or null if the slice does not have a label.
+	*/
+	public String getShortSliceLabel(int n, int max) {
 		String shortLabel = getSliceLabel(n);
 		if (shortLabel==null) return null;
     	int newline = shortLabel.indexOf('\n');
@@ -277,12 +288,12 @@ public class ImageStack {
     	int len = shortLabel.length();
 		if (len>4 && shortLabel.charAt(len-4)=='.' && !Character.isDigit(shortLabel.charAt(len-1)))
 			shortLabel = shortLabel.substring(0,len-4);
-		if (shortLabel.length()>60)
-			shortLabel = shortLabel.substring(0, 60);
+		if (shortLabel.length()>max)
+			shortLabel = shortLabel.substring(0, max);
 		return shortLabel;
 	}
-
-	/** Sets the label of the specified slice, were 1<=n<=nslices. */
+	
+	/** Sets the label of the specified slice, where {@literal 1<=n<=nslices}. */
 	public void setSliceLabel(String label, int n) {
 		if (n<1 || n>nSlices)
 			throw new IllegalArgumentException(outOfRange+n);
@@ -290,9 +301,10 @@ public class ImageStack {
 	}
 	
 	/** Returns an ImageProcessor for the specified slice,
-		were 1<=n<=nslices. Returns null if the stack is empty.
-	*/
-	public ImageProcessor getProcessor(int n) {
+	 * where {@literal 1<=n<=nslices}.
+ 	 * Returns null if the stack is empty.
+ 	*/
+ 	public ImageProcessor getProcessor(int n) {
 		ImageProcessor ip;
 		if (n<1 || n>nSlices)
 			throw new IllegalArgumentException(outOfRange+n);
@@ -318,11 +330,13 @@ public class ImageStack {
 			ip.setMinAndMax(min, max);
 		if (cTable!=null)
 			ip.setCalibrationTable(cTable);
+		ip.setSliceNumber(n);
 		return ip;
 	}
 	
-	/** Assigns the pixel array of an ImageProcessor to the
-		 specified slice, were 1<=n<=nslices. */
+	/** Assigns the pixel array of an ImageProcessor to the specified slice,
+	 * where {@literal 1<=n<=nslices}.
+	*/
 	public void setProcessor(ImageProcessor ip, int n) {
 		if (n<1 || n>nSlices)
 			throw new IllegalArgumentException(outOfRange+n);
@@ -698,7 +712,6 @@ public class ImageStack {
 	 
 	 int viewers(int inc) {
 	 	viewers += inc;
-	 	if (IJ.debugMode) IJ.log("stack.viewers: "+viewers);
 	 	return viewers;
 	 }
 	 
