@@ -58,7 +58,8 @@ public class Prefs {
 		REVERSE_NEXT_PREVIOUS_ORDER=1<<5, AUTO_RUN_EXAMPLES=1<<6, SHOW_ALL_POINTS=1<<7,
 		DO_NOT_SAVE_WINDOW_LOCS=1<<8, JFILE_CHOOSER_CHANGED=1<<9,
 		CANCEL_BUTTON_ON_RIGHT=1<<10, IGNORE_RESCALE_SLOPE=1<<11,
-		NON_BLOCKING_DIALOGS=1<<12;
+		NON_BLOCKING_DIALOGS=1<<12, FIXED_DICOM_SCALING=1<<13,
+		CALIBRATE_CONVERSIONS=1<<14;
 	public static final String OPTIONS2 = "prefs.options2";
     
 	/** file.separator system property */
@@ -90,7 +91,7 @@ public class Prefs {
 	/** Draw tool icons using antialiasing (always true). */
 	public static boolean antialiasedTools = true;
 	/** Export TIFF and Raw using little-endian byte order. */
-	public static boolean intelByteOrder;
+	public static boolean intelByteOrder = true;
 	/** No longer used */
 	public static boolean doubleBuffer = true;
 	/** Do not label multiple points created using point tool. */
@@ -100,7 +101,7 @@ public class Prefs {
 	/** Do not draw black border around image. */
 	public static boolean noBorder;
 	/** Only show ROIs associated with current slice in Roi Manager "Show All" mode. */
-	public static boolean showAllSliceOnly;
+	public static boolean showAllSliceOnly = true;
 	/** Include column headers when copying tables to clipboard. */
 	public static boolean copyColumnHeaders;
 	/** Do not include row numbers when copying tables to clipboard. */
@@ -121,6 +122,8 @@ public class Prefs {
 	public static boolean openDicomsAsFloat;
 	/** Ignore Rescale Slope when opening DICOMs */
 	public static boolean ignoreRescaleSlope;
+	/** Assume DICOM volumes use identical RescaleSlope and RescaleIntercept across all slices */	
+	public static boolean fixedDicomScaling;
 	/** Plot rectangular selectons vertically */
 	public static boolean verticalProfile;
 	/** Rotate YZ orthogonal views 90 degrees */
@@ -186,6 +189,13 @@ public class Prefs {
 	public static boolean supportMacroUndo;
 	/** Use NonBlockingGenericDialogs in filters */	
 	public static boolean nonBlockingFilterDialogs;
+	/** Turn live display on plots automatically */
+	public static boolean autoLivePlots;
+	/** Use full range for 16-bit inversions */
+	public static boolean fullRange16bitInversions;
+	/** Calibrate image type conversions */
+	public static boolean calibrateConversions;
+
 	//Save location of moved image windows */	
 	//public static boolean saveImageLocation = true;
 
@@ -195,6 +205,7 @@ public class Prefs {
 	static String prefsDir;
 	static String imagesURL;
 	static String ImageJDir;
+	static String pluginsDirProperty;
 	static int threads;
 	static int transparentIndex = -1;
 	private static boolean resetPreferences;
@@ -262,7 +273,7 @@ public class Prefs {
 		String s = ijPrefs.getProperty(KEY_PREFIX+key);
 		Double d = null;
 		if (s!=null) {
-			try {d = new Double(s);}
+			try {d = Double.valueOf(s);}
 			catch (NumberFormatException e) {d = null;}
 			if (d!=null)
 				return(d.doubleValue());
@@ -288,29 +299,31 @@ public class Prefs {
 	public static String load(Object ij, Applet applet) {
 		if (ImageJDir==null)
 			ImageJDir = System.getProperty("user.dir");
-		InputStream f = null;
-		try { // Look for IJ_Props.txt in ImageJ folder
-			f = new FileInputStream(ImageJDir+"/"+PROPS_NAME);
-			propertiesPath = ImageJDir+"/"+PROPS_NAME;
-		} catch (FileNotFoundException e) {
-			f = null;
+		if (ij!=null) {
+			InputStream f = null;
+			try { // Look for IJ_Props.txt in ImageJ folder
+				f = new FileInputStream(ImageJDir+"/"+PROPS_NAME);
+				propertiesPath = ImageJDir+"/"+PROPS_NAME;
+			} catch (FileNotFoundException e) {
+				f = null;
+			}
+			if (f==null) {
+				// Look in ij.jar if not found in ImageJ folder
+				f = ij.getClass().getResourceAsStream("/"+PROPS_NAME);
+			}			
+			if (applet!=null)
+				return loadAppletProps(f, applet);
+			if (f==null)
+				return PROPS_NAME+" not found in ij.jar or in "+ImageJDir;
+			f = new BufferedInputStream(f);
+			try {
+				props.load(f);
+				f.close();
+			} catch (IOException e) {
+				return("Error loading "+PROPS_NAME);
+			}
+			imagesURL = props.getProperty(IJ.isJava18()?"images.location":"images.location2");
 		}
-		if (f==null) {
-			// Look in ij.jar if not found in ImageJ folder
-			f = ij.getClass().getResourceAsStream("/"+PROPS_NAME);
-		}			
-		if (applet!=null)
-			return loadAppletProps(f, applet);
-		if (f==null)
-			return PROPS_NAME+" not found in ij.jar or in "+ImageJDir;
-		f = new BufferedInputStream(f);
-		try {
-			props.load(f);
-			f.close();
-		} catch (IOException e) {
-			return("Error loading "+PROPS_NAME);
-		}
-		imagesURL = props.getProperty(IJ.isJava18()?"images.location":"images.location2");
 		loadPreferences();
 		loadOptions();
 		guiScale = get(GUI_SCALE, 1.0);
@@ -363,24 +376,54 @@ public class Prefs {
 	/** Returns the path, ending in File.separator, to the ImageJ directory. */
 	public static String getImageJDir() {
 		String path = Menus.getImageJPath();
-		if (path==null)
-			return ImageJDir + File.separator;
-		else
+		if (path==null) {
+			String ijPath = getPluginsDirProperty();
+			//if (ijPath==null)
+			//	ijPath = ImageJDir;
+			if (ijPath==null)
+				ijPath = System.getProperty("user.dir");
+			return ijPath + File.separator;
+		} else
 			return path;
 	}
+	
+	public static String getPluginsDirProperty() {
+		if (pluginsDirProperty==null) {
+			String ijDir = System.getProperty("plugins.dir");
+			if (ijDir!=null) {
+				if (ijDir.endsWith("/")||ijDir.endsWith("\\"))
+					ijDir = ijDir.substring(0, ijDir.length()-1);
+				if (ijDir.endsWith("/plugins")||ijDir.endsWith("\\plugins"))
+					ijDir = ijDir.substring(0, ijDir.length()-8);
+				pluginsDirProperty = ijDir;
+			} else
+				pluginsDirProperty = "";
+		}
+		return pluginsDirProperty.length()>0?pluginsDirProperty:null;
+	}
 
 	/** Returns the path to the directory where the 
 		preferences file (IJPrefs.txt) is saved. */
 	public static String getPrefsDir() {
+		// look in current directory
 		if (prefsDir==null) {
-			if (ImageJDir==null)
-				ImageJDir = System.getProperty("user.dir");
-			File f = new File(ImageJDir+File.separator+PREFS_NAME);
+			String cwd = System.getProperty("user.dir");
+			File f = new File(cwd+File.separator+PREFS_NAME);
 			if (f.exists()) {
-				prefsDir = ImageJDir;
-				preferencesPath = ImageJDir+"/"+PREFS_NAME;
+				prefsDir = cwd;
+				preferencesPath = cwd+"/"+PREFS_NAME;
 			}
-			//System.out.println("getPrefsDir: "+f+"  "+prefsDir);
+			// look in ImageJ directory
+			if (prefsDir==null) {
+				String ijDir = getImageJDir();
+				ijDir = ijDir.substring(0, ijDir.length()-1);
+				f = new File(ijDir+File.separator+PREFS_NAME);
+				if (f.exists()) {
+					prefsDir = ijDir;
+					preferencesPath = ijDir+"/"+PREFS_NAME;
+				}
+			}
+			// use home directory
 			if (prefsDir==null) {
 				String dir = System.getProperty("user.home");
 				if (IJ.isMacOSX())
@@ -389,7 +432,7 @@ public class Prefs {
 					dir += File.separator+".imagej";
 				prefsDir = dir;
 			}
-		}
+		}		
 		return prefsDir;
 	}
 
@@ -443,6 +486,7 @@ public class Prefs {
 	/** Saves user preferences in the IJ_Prefs.txt properties file. */
 	public static void savePreferences() {
 		String path = null;
+		commandLineMacro = false;
 		try {
 			Properties prefs = new Properties();
 			String dir = OpenDialog.getDefaultDirectory();
@@ -461,7 +505,9 @@ public class Prefs {
 			if (!IJ.isLinux()) dialogCancelButtonOnRight = false;
 			saveOptions(prefs);
 			savePluginPrefs(prefs);
-			IJ.getInstance().savePreferences(prefs);
+			ImageJ ij = IJ.getInstance();
+			if (ij!=null)
+				ij.savePreferences(prefs);
 			Menus.savePreferences(prefs);
 			ParticleAnalyzer.savePreferences(prefs);
 			Analyzer.savePreferences(prefs);
@@ -507,7 +553,7 @@ public class Prefs {
 		antialiasedText = false;
 		interpolateScaledImages = (options&INTERPOLATE)!=0;
 		open100Percent = (options&ONE_HUNDRED_PERCENT)!=0;
-		//blackBackground = (options&BLACK_BACKGROUND)!=0;
+		blackBackground = (options&BLACK_BACKGROUND)!=0;
 		useJFileChooser = (options&JFILE_CHOOSER)!=0;
 		weightedColor = (options&WEIGHTED)!=0;
 		if (weightedColor)
@@ -517,7 +563,7 @@ public class Prefs {
 		useInvertingLut = (options&USE_INVERTING_LUT)!=0;
 		intelByteOrder = (options&INTEL_BYTE_ORDER)!=0;
 		noBorder = (options&NO_BORDER)!=0;
-		showAllSliceOnly = (options&SHOW_ALL_SLICE_ONLY)!=0;
+		//showAllSliceOnly = (options&SHOW_ALL_SLICE_ONLY)!=0;
 		copyColumnHeaders = (options&COPY_HEADERS)!=0;
 		noRowNumbers = (options&NO_ROW_NUMBERS)!=0;
 		moveToMisc = (options&MOVE_TO_MISC)!=0;
@@ -546,6 +592,8 @@ public class Prefs {
 		dialogCancelButtonOnRight = (options2&CANCEL_BUTTON_ON_RIGHT)!=0;
 		ignoreRescaleSlope = (options2&IGNORE_RESCALE_SLOPE)!=0;
 		nonBlockingFilterDialogs = (options2&NON_BLOCKING_DIALOGS)!=0;
+		fixedDicomScaling = (options2&FIXED_DICOM_SCALING)!=0;
+		//calibrateConversions = (options2&CALIBRATE_CONVERSIONS)!=0;
 	}
 
 	static void saveOptions(Properties prefs) {
@@ -576,7 +624,9 @@ public class Prefs {
 			+ (jFileChooserSettingChanged?JFILE_CHOOSER_CHANGED:0)
 			+ (dialogCancelButtonOnRight?CANCEL_BUTTON_ON_RIGHT:0)
 			+ (ignoreRescaleSlope?IGNORE_RESCALE_SLOPE:0)
-			+ (nonBlockingFilterDialogs?NON_BLOCKING_DIALOGS:0);
+			+ (nonBlockingFilterDialogs?NON_BLOCKING_DIALOGS:0)
+			+ (fixedDicomScaling?FIXED_DICOM_SCALING:0);
+			//+ (calibrateConversions?CALIBRATE_CONVERSIONS:0);
 		prefs.put(OPTIONS2, Integer.toString(options2));
 	}
 
@@ -732,7 +782,7 @@ public class Prefs {
 		String s = props.getProperty(key);
 		Double d = null;
 		if (s!=null) {
-			try {d = new Double(s);}
+			try {d = Double.valueOf(s);}
 			catch (NumberFormatException e){d = null;}
 			if (d!=null)
 				return(d.doubleValue());
@@ -758,6 +808,10 @@ public class Prefs {
 			return defaultColor;
 		return new Color((i >> 16) & 0xFF, (i >> 8) & 0xFF, i & 0xFF);
 	}
+	
+	public static boolean commandLineMacro() {
+		return commandLineMacro;
+	}
 
 }
 
