@@ -267,7 +267,8 @@ public class PlotContentsDialog implements DialogListener {
 	}
 
 	public boolean dialogItemChanged(GenericDialog gd, AWTEvent e) {
-		if (e == null) return true;	//gets called with e=null upon OK
+		if (e==null)
+			return true;	//gets called with e=null upon OK
 		boolean setStyle = false;
 		if (dialogType == STYLE) {
 			int objectIndex = objectChoice.getSelectedIndex();  // no getNextChoice since Choices depend on dialog type
@@ -442,7 +443,7 @@ public class PlotContentsDialog implements DialogListener {
 		for (int i=0; i<allTables.length; i++) {
 			TextWindow tw = tableWindows.get(i);
 			allTables[i] = tw.getResultsTable();
-			if (allTableNames[i] == previousTableName)
+			if (allTableNames[i]!=null && allTableNames[i].equals(previousTableName))
 				defaultTableIndex = i;
 			allTableNames[i] = tw.getTitle();
 		}
