@@ -266,6 +266,8 @@ public class PlotWindow extends ImageWindow implements ActionListener, ItemListe
 			showList(/*useLabels=*/false);
 		else
 			ic.requestFocus();	//have focus on the canvas, not the button, so that pressing the space bar allows panning
+		if (Prefs.autoLivePlots && bgThread==null)
+			enableLivePlot();
 	}
 
 	/** Sets the Plot object shown in this PlotWindow. Does not update the window. */
@@ -398,7 +400,7 @@ public class PlotWindow extends ImageWindow implements ActionListener, ItemListe
 		try {
 		Object b = e.getSource();
 		if (b==live)
-			toggleLiveProfiling();
+			toggleLivePlot();
 		else if (b==list)
 			showList(/*useLabels=*/true);
 		else if (b==data) {
@@ -630,7 +632,7 @@ public class PlotWindow extends ImageWindow implements ActionListener, ItemListe
         rangeArrowsVisible = true;
     }
 
-	void hideRangeArrows() {
+	public void hideRangeArrows() {
 		if (imp == null || rangeArrowRois==null) return;
 		Overlay ovly = imp.getOverlay();
 		if (ovly == null) return;
@@ -784,16 +786,25 @@ public class PlotWindow extends ImageWindow implements ActionListener, ItemListe
 		prefs.put(OPTIONS, Integer.toString(options));
 	}
 
-	private void toggleLiveProfiling() {
-		boolean liveMode = bgThread != null;
-		if (liveMode)
-			disableLivePlot();
-		else
+	/** Enables or disables live plotting.
+	 *  Does nothing if the plot has no PlotMaker (then there is no 'live' button). */
+	public void enableLivePlot(boolean b) {
+		if (plot==null || plot.getPlotMaker()==null || live == null)
+			return;
+		if (b && bgThread == null)
 			enableLivePlot();
+		else if (!b && bgThread != null)
+			disableLivePlot();
 	}
 
-	/* Enable live plotting.
-	 * This requires that the PlotWindow has been initialized with a Plot having a PlotMaker */
+	private void toggleLivePlot() {
+		boolean liveMode = bgThread != null;
+		enableLivePlot(!liveMode);
+	}
+
+	/** Enables live plotting.
+	 *  This requires that the PlotWindow has been initialized with a Plot having a PlotMaker.
+	 *  Live plotting runs in a background thread. */
 	private void enableLivePlot() {
 		if (plotMaker==null)
 			plotMaker = plot!=null?plot.getPlotMaker():null;
@@ -815,6 +826,7 @@ public class PlotWindow extends ImageWindow implements ActionListener, ItemListe
 		live.setForeground(Color.red);
 	}
 
+	/** Disables live plotting (when currently active) */
 	private void disableLivePlot() {
 		if (IJ.debugMode) IJ.log("PlotWindow.disableLivePlot: "+srcImp);
 		if (srcImp==null)
