@@ -13,7 +13,9 @@ import java.awt.event.KeyEvent;
 import java.util.*;
 import java.awt.geom.*;
 
-/** This class represents a collection of points that can be associated with counters.
+/** This class represents a collection of points that can be associated 
+ * with counters. Use the getPolygon() or getFloatPolygon() methods
+ * to retrieve the coordinates of the points. 
  * @see <a href="http://wsr.imagej.net/macros/js/PointProperties.js">PointProperties.js</a>
 */
 public class PointRoi extends PolygonRoi {
@@ -52,28 +54,26 @@ public class PointRoi extends PolygonRoi {
 	private int nMarkers;
 	private boolean addToOverlay;
 	public static PointRoi savedPoints;
-		
+
 	static {
 		setDefaultType((int)Prefs.get(TYPE_KEY, HYBRID));
 		setDefaultSize((int)Prefs.get(SIZE_KEY, 1));
 	}
-	
+
 	public PointRoi() {
 		this(0.0, 0.0);
 		deletePoint(0);
 	}
-	
+
 	/** Creates a new PointRoi using the specified int arrays of offscreen coordinates. */
 	public PointRoi(int[] ox, int[] oy, int points) {
 		super(itof(ox), itof(oy), points, POINT);
-		width+=1; height+=1;
 		updateCounts();
 	}
 
 	/** Creates a new PointRoi using the specified float arrays of offscreen coordinates. */
 	public PointRoi(float[] ox, float[] oy, int points) {
 		super(ox, oy, points, POINT);
-		width+=1; height+=1;
 		updateCounts();
 	}
 
@@ -142,14 +142,14 @@ public class PointRoi extends PolygonRoi {
 		if (Recorder.record) {
 			String add = Prefs.pointAddToOverlay?" add":"";
 			String options = sizes[convertSizeToIndex(size)]+" "+Colors.colorToString(getColor())+" "+types[type]+add;
-			options = options.toLowerCase();		
+			options = options.toLowerCase();
 			if (Recorder.scriptMode())
 				Recorder.recordCall("imp.setRoi(new PointRoi("+x+","+y+",\""+options+"\"));");
 			else
 				Recorder.record("makePoint", x, y, options);
 		}
 	}
-	
+
 	public void setOptions(String options) {
 		if (options==null)
 			return;
@@ -167,7 +167,7 @@ public class PointRoi extends PolygonRoi {
 		setStrokeColor(Colors.getColor(options,Roi.getColor()));
 		addToOverlay =  options.contains("add");
 	}
-	
+
 	static float[] itof(int[] arr) {
 		if (arr==null)
 			return null;
@@ -192,15 +192,15 @@ public class PointRoi extends PolygonRoi {
 		}
 		return new float[] {(float)value};
 	}
-				
+
 	void handleMouseMove(int ox, int oy) {
 	}
-	
+
 	protected void handleMouseUp(int sx, int sy) {
 		super.handleMouseUp(sx, sy);
 		modifyRoi(); //adds this point to previous points if shift key down
 	}
-	
+
 	/** Draws the points on the image. */
 	public void draw(Graphics g) {
 		updatePolygon();
@@ -215,7 +215,7 @@ public class PointRoi extends PolygonRoi {
 			if (fontSize>9)
 				Java2.setAntialiasedText(g, true);
 		}
-		int slice = imp!=null&&positions!=null&&imp.getStackSize()>1?imp.getCurrentSlice():0;		
+		int slice = imp!=null&&positions!=null&&imp.getStackSize()>1?imp.getCurrentSlice():0;
 		ImageCanvas ic = imp!=null?imp.getCanvas():null;
 		if (ic!=null && overlay && ic.getShowAllList()!=null && ic.getShowAllList().contains(this) && !Prefs.showAllSliceOnly)
 			slice = 0;  // draw point irrespective of currently selected slice
@@ -271,7 +271,7 @@ public class PointRoi extends PolygonRoi {
 		}
 		if (type!=CROSS && size>SMALL)
 			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
-		if (type==HYBRID || type==DOT) { 
+		if (type==HYBRID || type==DOT) {
 			if (!colorSet) {
 				g.setColor(color);
 				colorSet = true;
@@ -323,7 +323,7 @@ public class PointRoi extends PolygonRoi {
 		if (saveXform!=null)
 			g2d.setTransform(saveXform);
 	}
-	
+
 	public void drawPixels(ImageProcessor ip) {
 		ip.setLineWidth(Analyzer.markWidth);
 		double x0 = bounds == null ? x : bounds.x;
@@ -333,16 +333,15 @@ public class PointRoi extends PolygonRoi {
 			ip.lineTo((int)Math.round(x0+xpf[i]), (int)Math.round(y0+ypf[i]));
 		}
 	}
-	
+
 	/** Adds a point to this PointRoi. */
 	public void addPoint(ImagePlus imp, double ox, double oy) {
 		if (nPoints==xpf.length)
 			enlargeArrays();
 		addPoint2(imp, ox, oy);
 		resetBoundingRect();
-		width+=1; height+=1;
 	}
-	
+
 
 	public void addUserPoint(ImagePlus imp, double ox, double oy) {
 		addPoint(imp, ox, oy);
@@ -360,13 +359,25 @@ public class PointRoi extends PolygonRoi {
 		incrementCounter(imp);
 		lastPointTime = System.currentTimeMillis();
 	}
-	
+
 	/** Adds a point to this PointRoi. */
 	public PointRoi addPoint(double x, double y) {
 		addPoint(null, x, y);
 		return this;
 	}
 
+	/** Adds a point at the specified stack position. */
+	public void addPoint(double x, double y, int position) {
+		if (counters==null) {
+			int size = nPoints*2;
+			if (size<100) size=100;
+			counters = new short[size];
+			positions = new int[size];
+		}
+		addPoint(null, x, y);
+		positions[nPoints-1] = position;	
+	}
+
 	protected void deletePoint(int index) {
 		super.deletePoint(index);
 		if (index>=0 && index<=nPoints && counters!=null) {
@@ -377,7 +388,9 @@ public class PointRoi extends PolygonRoi {
 			}
 			if (rt!=null && WindowManager.getFrame(getCountsTitle())!=null)
 				displayCounts();
-		}
+		} else if (index==0 && nPoints==0)
+			counts[0] = 0;
+			
 	}
 
 	private synchronized void incrementCounter(ImagePlus imp) {
@@ -386,15 +399,12 @@ public class PointRoi extends PolygonRoi {
 		boolean isStack = imp!=null && imp.getStackSize()>1;
 		if (counter!=0 || isStack || counters!=null) {
 			if (counters==null) {
-				counters = new short[nPoints*2];
-				positions = new int[nPoints*2];
+				int size = nPoints*2;
+				if (size<100) size=100;
+				counters = new short[size];
+				positions = new int[size];
 			}
-			counters[nPoints-1] = (short)counter;
-			if (imp!=null)
-					positions[nPoints-1] = imp.getStackSize()>1 ? imp.getCurrentSlice() : 0;
-			//if (positions[nPoints-1]==0 || positions[nPoints-1]==1 || counters[nPoints-1]==0)
-			//	IJ.log("incrementCounter: "+nPoints+" "+" "+positions[nPoints-1]+" "+counters[nPoints-1]+" "+imp);
-			if (nPoints+1==counters.length) {
+			if (nPoints>=counters.length) {
 				short[] temp = new short[counters.length*2];
 				System.arraycopy(counters, 0, temp, 0, counters.length);
 				counters = temp;
@@ -402,6 +412,11 @@ public class PointRoi extends PolygonRoi {
 				System.arraycopy(positions, 0, temp1, 0, positions.length);
 				positions = temp1;
 			}
+			counters[nPoints-1] = (short)counter;
+			if (imp!=null)
+					positions[nPoints-1] = imp.getStackSize()>1 ? imp.getCurrentSlice() : 0;
+			//if (positions[nPoints-1]==0 || positions[nPoints-1]==1 || counters[nPoints-1]==0)
+			//IJ.log("incrementCounter: "+nPoints+" "+counters.length+" "+counters[nPoints-1]+"   "+positions[nPoints-1]+" "+imp);
 		}
 		if (rt!=null && WindowManager.getFrame(getCountsTitle())!=null)
 			displayCounts();
@@ -422,7 +437,7 @@ public class PointRoi extends PolygonRoi {
 		else
 			return counts[counter];
 	}
-	
+
 	/** Returns the index of the last counter. */
 	public int getLastCounter() {
 		return nCounters - 1;
@@ -436,7 +451,7 @@ public class PointRoi extends PolygonRoi {
 		}
 		return n;
 	}
-	
+
 	/** Returns the counter assocated with the specified point. */
 	public int getCounter(int index) {
 		if (counters==null || index>=counters.length)
@@ -452,7 +467,7 @@ public class PointRoi extends PolygonRoi {
 		positions = null;
 		PointToolOptions.update();
 	}
-	
+
 	/** Returns the points of this Roi that are not contained in the specified area ROI.
 	 *  Returns null if there are no resulting points or Roi is not an area roi. */
 	public PointRoi subtractPoints(Roi roi) {
@@ -491,7 +506,7 @@ public class PointRoi extends PolygonRoi {
 		double deltaX = getXBase() - x;
 		double deltaY = getYBase() - y;
 		for (int i=0; i<nPoints; i++)
-			mask.putPixel((int)(xpf[i] + deltaX), (int)(ypf[i] + deltaY), 255);
+			mask.putPixel((int)Math.round(xpf[i] + deltaX), (int)Math.round(ypf[i] + deltaY), 255);
 		cachedMask = mask;
 		return mask;
 	}
@@ -503,7 +518,7 @@ public class PointRoi extends PolygonRoi {
 		}
 		return false;
 	}
-	
+
 	public void setShowLabels(boolean showLabels) {
 		this.showLabels = showLabels;
 	}
@@ -521,11 +536,11 @@ public class PointRoi extends PolygonRoi {
 			Prefs.set(TYPE_KEY, type);
 		}
 	}
-	
+
 	public static int getDefaultType() {
 		return defaultType;
 	}
-	
+
 	/** Sets the point type (0=hybrid, 1=cross, 2=dot, 3=circle). */
 	public void setPointType(int type) {
 		if (type>=0 && type<types.length)
@@ -549,7 +564,7 @@ public class PointRoi extends PolygonRoi {
 			Prefs.set(SIZE_KEY, index);
 		}
 	}
-	
+
 	/** Returns the default point size 0-6 (Tiny-XXXL). */
 	public static int getDefaultSize() {
 		return convertSizeToIndex(defaultSize);
@@ -565,7 +580,7 @@ public class PointRoi extends PolygonRoi {
 	public int getSize() {
 		return convertSizeToIndex(size);
 	}
-	
+
 	private static int convertSizeToIndex(int size) {
 		switch (size) {
 			case TINY: return 0;
@@ -596,7 +611,7 @@ public class PointRoi extends PolygonRoi {
 	public boolean subPixelResolution() {
 		return true;
 	}
-	
+
 	private static PointRoi getPointRoiInstance() {
 		ImagePlus imp = WindowManager.getCurrentImage();
 		if (imp!=null) {
@@ -620,7 +635,7 @@ public class PointRoi extends PolygonRoi {
 	    	return promptBeforeDeleting;
 	    else
 			return (nMarkers>8||getNCounters()>1) && imp!=null && imp.getWindow()!=null;
-	} 
+	}
 
 	public void promptBeforeDeleting(Boolean prompt) {
 		promptBeforeDeleting = prompt;
@@ -682,7 +697,7 @@ public class PointRoi extends PolygonRoi {
 		else
 			return 0;
 	}
-	
+
 	public void displayCounts() {
 		ImagePlus imp = getImage();
 		String firstColumnHdr = "Slice";
@@ -692,8 +707,7 @@ public class PointRoi extends PolygonRoi {
 			int nChannels = 1;
 			int nSlices = 1;
 			int nFrames = 1;
-			boolean isHyperstack = false;				isHyperstack = true;
-
+			boolean isHyperstack = true;
 			if (imp.isComposite() || imp.isHyperStack()) {
 				nChannels = imp.getNChannels();
 				nSlices = imp.getNSlices();
@@ -753,7 +767,7 @@ public class PointRoi extends PolygonRoi {
 		rt.show(getCountsTitle());
 		if (IJ.debugMode) debug();
 	}
-	
+
 	private void debug() {
 		FloatPolygon p = getFloatPolygon();
 		ResultsTable rt = new ResultsTable();
@@ -761,7 +775,7 @@ public class PointRoi extends PolygonRoi {
 			if (counters!=null) {
 				rt.setValue("Counter", i, counters[i]);
 				rt.setValue("Position", i, positions[i]);
-			} 
+			}
 			rt.setValue("X", i, p.xpoints[i]);
 			rt.setValue("Y", i, p.ypoints[i]);
 		}
@@ -771,7 +785,7 @@ public class PointRoi extends PolygonRoi {
 	private String getCountsTitle() {
 		return "Counts_"+(imp!=null?imp.getTitle():"");
 	}
-	
+
 	public synchronized static String[] getCounterChoices() {
 		if (counterChoices==null) {
 			counterChoices = new String[MAX_COUNTERS];
@@ -780,7 +794,7 @@ public class PointRoi extends PolygonRoi {
 		}
 		return counterChoices;
 	}
-	
+
 	private static Color getColor(int index) {
 		if (colors==null) {
 			colors = new Color[MAX_COUNTERS];
@@ -803,7 +817,7 @@ public class PointRoi extends PolygonRoi {
 	}
 
 	/** Returns a point index if it has been at least one second since
-		the last point was added and the specified screen coordinates are	 
+		the last point was added and the specified screen coordinates are
 		inside or near a point, otherwise returns -1. */
 	public int isHandle(int sx, int sy) {
 		if ((System.currentTimeMillis()-lastPointTime)<1000L)
@@ -824,15 +838,15 @@ public class PointRoi extends PolygonRoi {
 		}
 		return handle;
 	}
-	
-	/** Returns the points as an array of Points. 
+
+	/** Returns the points as an array of Points.
 	 * Wilhelm Burger: modified to use FloatPolygon for correct point positions.
 	*/
 	public Point[] getContainedPoints() {
 		FloatPolygon p = getFloatPolygon();
 		Point[] points = new Point[p.npoints];
 		for (int i=0; i<p.npoints; i++)
-			points[i] = new Point((int) Math.round(p.xpoints[i] - 0.5f), (int) Math.round(p.ypoints[i] - 0.5f));
+			points[i] = new Point((int) Math.round(p.xpoints[i]), (int) Math.round(p.ypoints[i]));
 		return points;
 	}
 
@@ -845,7 +859,7 @@ public class PointRoi extends PolygonRoi {
 	 * Custom iterator for points contained in a {@link PointRoi}.
 	 * Author: W. Burger
 	*/
-	public Iterator<Point> iterator() {	
+	public Iterator<Point> iterator() {
 		return new Iterator<Point>() {
 			final Point[] pnts = getContainedPoints();
 			final int n = pnts.length;
@@ -869,7 +883,7 @@ public class PointRoi extends PolygonRoi {
 			}
 		};
 	}
-	
+
 	@Override
 	protected int getClosestPoint(double x, double y, FloatPolygon points) {
 		int index = -1;
@@ -909,7 +923,19 @@ public class PointRoi extends PolygonRoi {
 		}
 		return r;
 	}
-	
+
+	/* Returns a version of this PointRoi that only contains points inside 'roi'. */
+	public PointRoi crop(Roi roi) {
+		PointRoi points = (PointRoi)clone();
+		Polygon p = points.getPolygon();
+		for (int i=points.size()-1; i>=0; i--) {
+			if (!roi.contains(p.xpoints[i],p.ypoints[i])) {
+				points.deletePoint(i);
+			}
+		}
+		return points;
+	}
+
 	@Override
 	public void copyAttributes(Roi roi2) {
 		super.copyAttributes(roi2);
@@ -921,7 +947,7 @@ public class PointRoi extends PolygonRoi {
 			this.fontSize = p2.fontSize;
 		}
 	}
-	
+
 	public void setCounterInfo(int[] info) {
 		counterInfo = info;
 	}
@@ -929,7 +955,7 @@ public class PointRoi extends PolygonRoi {
 	public int[] getCounterInfo() {
 		return counterInfo;
 	}
-	
+
 	public boolean addToOverlay() {
 		return addToOverlay;
 	}
@@ -940,25 +966,25 @@ public class PointRoi extends PolygonRoi {
 		else
 			return ("Roi[Point, x="+x+", y="+y+"]");
 	}
-	
+
 	/** @deprecated */
 	public void setHideLabels(boolean hideLabels) {
 		this.showLabels = !hideLabels;
 	}
-	
+
 	/** @deprecated */
 	public static void setDefaultMarkerSize(String size) {
 	}
-	
+
 	/** @deprecated */
 	public static String getDefaultMarkerSize() {
 		return sizes[defaultSize];
 	}
-	
+
 	/** Deprecated */
 	public static void setDefaultCrossColor(Color color) {
 	}
-	
+
 	/** Deprecated */
 	public static Color getDefaultCrossColor() {
 		return null;
