@@ -83,6 +83,7 @@ public class FileOpener {
 			case FileInfo.GRAY16_SIGNED:
 			case FileInfo.GRAY16_UNSIGNED:
 			case FileInfo.GRAY12_UNSIGNED:
+			case FileInfo.GRAY10_UNSIGNED:
 				pixels = readPixels(fi);
 				if (pixels==null) return null;
 	    		ip = new ShortProcessor(width, height, (short[])pixels, cm);
@@ -159,7 +160,7 @@ public class FileOpener {
 		if (fi.info!=null)
 			imp.setProperty("Info", fi.info);
 		if (fi.sliceLabels!=null&&fi.sliceLabels.length==1&&fi.sliceLabels[0]!=null)
-			imp.setProperty("Label", fi.sliceLabels[0]);
+			imp.setProp("Slice_Label", fi.sliceLabels[0]);
 		if (fi.plot!=null) try {
 			Plot plot = new Plot(imp, new ByteArrayInputStream(fi.plot));
 			imp.setProperty(Plot.PROPERTY_KEY, plot);
@@ -174,6 +175,53 @@ public class FileOpener {
 		return imp;
 	}
 	
+	public ImageProcessor openProcessor() {
+		Object pixels;
+		ProgressBar pb=null;
+		ImageProcessor ip = null;		
+		ColorModel cm = createColorModel(fi);
+		switch (fi.fileType) {
+			case FileInfo.GRAY8:
+			case FileInfo.COLOR8:
+			case FileInfo.BITMAP:
+				pixels = readPixels(fi);
+				if (pixels==null) return null;
+				ip = new ByteProcessor(width, height, (byte[])pixels, cm);
+				break;
+			case FileInfo.GRAY16_SIGNED:
+			case FileInfo.GRAY16_UNSIGNED:
+			case FileInfo.GRAY12_UNSIGNED:
+			case FileInfo.GRAY10_UNSIGNED:
+				pixels = readPixels(fi);
+				if (pixels==null) return null;
+	    		ip = new ShortProcessor(width, height, (short[])pixels, cm);
+				break;
+			case FileInfo.GRAY32_INT:
+			case FileInfo.GRAY32_UNSIGNED:
+			case FileInfo.GRAY32_FLOAT:
+			case FileInfo.GRAY24_UNSIGNED:
+			case FileInfo.GRAY64_FLOAT:
+				pixels = readPixels(fi);
+				if (pixels==null) return null;
+	    		ip = new FloatProcessor(width, height, (float[])pixels, cm);
+				break;
+			case FileInfo.RGB:
+			case FileInfo.BGR:
+			case FileInfo.ARGB:
+			case FileInfo.ABGR:
+			case FileInfo.BARG:
+			case FileInfo.RGB_PLANAR:
+			case FileInfo.CMYK:
+				pixels = readPixels(fi);
+				if (pixels==null) return null;
+				ip = new ColorProcessor(width, height, (int[])pixels);
+				if (fi.fileType==FileInfo.CMYK)
+					ip.invert();
+				break;
+		}
+		return ip;
+	}
+
 	void setOverlay(ImagePlus imp, byte[][] rois) {
 		Overlay overlay = new Overlay();
 		Overlay proto = null;
@@ -313,6 +361,7 @@ public class FileOpener {
 				case FileInfo.GRAY16_SIGNED:
 				case FileInfo.GRAY16_UNSIGNED:
 				case FileInfo.GRAY12_UNSIGNED:
+				case FileInfo.GRAY10_UNSIGNED:
 					ip = new ShortProcessor(width, height, (short[])pixels, cm);
 					imp.setProcessor(null, ip);
 					break;
@@ -347,7 +396,10 @@ public class FileOpener {
 		Calibration cal = imp.getCalibration();
 		boolean calibrated = false;
 		if (fi.pixelWidth>0.0 && fi.unit!=null) {
-			if (Prefs.convertToMicrons && fi.pixelWidth<=0.0001 && fi.unit.equals("cm")) {
+			double threshold = 0.001;
+			if (fi.description!=null && fi.description.startsWith("ImageJ"))
+				threshold = 0.0001;
+			if (Prefs.convertToMicrons && fi.pixelWidth<=threshold && fi.unit.equals("cm")) {
 				fi.pixelWidth *= 10000.0;
 				fi.pixelHeight *= 10000.0;
 				if (fi.pixelDepth!=1.0)
