@@ -101,15 +101,18 @@ public class FileSaver {
 		if (imp.getProperty("FHT")!=null && path.contains("FFT of "))
 			setupFFTSave();
 		fi.info = imp.getInfoProperty();
-		Object label = imp.getProperty("Label");
-		if (label!=null && (label instanceof String)) {
+		String label = imp.hasImageStack()?imp.getStack().getSliceLabel(1):null;
+		if (label!=null) {
 			fi.sliceLabels = new String[1];
-			fi.sliceLabels[0] = (String)label;
+			fi.sliceLabels[0] = label;
 		}
 		fi.description = getDescriptionString();
 		if (imp.getProperty(Plot.PROPERTY_KEY) != null) {
 			Plot plot = (Plot)(imp.getProperty(Plot.PROPERTY_KEY));
 			fi.plot = plot.toByteArray();
+			ImageWindow win = imp.getWindow();
+			if (win!=null && (win instanceof PlotWindow))
+				((PlotWindow)win).hideRangeArrows();
 		}
 		fi.roi = RoiEncoder.saveAsByteArray(imp.getRoi());
 		fi.overlay = getOverlay(imp);
@@ -161,6 +164,8 @@ public class FileSaver {
 			return null;
 		if (Orthogonal_Views.isOrthoViewsImage(imp))
 			return null;
+		if (n==1 && overlay.get(0).isCursor())
+			return null; // don't save red sync windows cursor
 		byte[][] array = new byte[n][];
 		for (int i=0; i<overlay.size(); i++) {
 			Roi roi = overlay.get(i);
@@ -686,6 +691,7 @@ public class FileSaver {
 			//	fi.offset = TiffEncoder.IMAGE_START;
 			fi.description = null;
 			imp.setTitle(name);
+			fi.imageSaved = true;
 			imp.setFileInfo(fi);
 		}
 	}
