@@ -1,6 +1,7 @@
 package ij.measure;
 import ij.*;
 import ij.plugin.filter.Analyzer;
+import ij.plugin.frame.Editor;
 import ij.text.*;
 import ij.process.*;
 import ij.gui.Roi;
@@ -67,6 +68,7 @@ public class ResultsTable implements Cloneable {
 	private boolean columnDeleted;
 	private boolean renameWhenSaving;
 	private boolean saveColumnHeaders = !Prefs.dontSaveHeaders;
+	public boolean isResultsTable;
 
 
 	/** Constructs an empty ResultsTable with the counter=0, no columns
@@ -105,6 +107,17 @@ public class ResultsTable implements Cloneable {
 		else
 			return null;
 	}
+	
+	/** Returns the active (front most) displayed ResultsTable. */
+	public static ResultsTable getActiveTable() {
+		ResultsTable rt = null;
+		Window win = WindowManager.getActiveTable();
+		if (win!=null && (win instanceof TextWindow)) {
+			TextPanel tp = ((TextWindow)win).getTextPanel();
+			rt = tp.getOrCreateResultsTable();
+		}
+		return rt;
+	}
 		
 	/** Obsolete. */
 	public static TextWindow getResultsWindow() {
@@ -217,7 +230,7 @@ public class ResultsTable implements Cloneable {
 	 *    rt.addValue("Cos(n)", Math.cos(n));
 	 * }
 	 * rt.show("Sine/Cosine Table");
-	 * <pre>
+	 * </pre>
 	 * @see #addRow
 	 * @see #addValue(String,String)
 	 * @see #size
@@ -291,6 +304,32 @@ public class ResultsTable implements Cloneable {
 			rowLabels = null;
 	}
 	
+	/** Returns a copy of the given column as a double array,
+		or null if the column is not found. */
+	public double[] getColumn(String column) {
+		int col = getColumnIndex(column);
+		if (col==COLUMN_NOT_FOUND || columns[col]==null)
+			throw new IllegalArgumentException("\""+column+"\" column not found");
+		return getColumnAsDoubles(col);
+	}
+
+	/** Returns a copy of the given column as a String array,
+		or null if the column is not found. */
+	public String[] getColumnAsStrings(String column) {
+		String[] array = new String[size()];
+		if ("Label".equals(column) && rowLabels!=null) {
+			for (int i=0; i<size(); i++)
+				array[i] = getLabel(i);
+			return array;
+		}
+		int col = getColumnIndex(column);
+		if (col==COLUMN_NOT_FOUND || columns[col]==null)
+			throw new IllegalArgumentException("\""+column+"\" column not found");
+		for (int i=0; i<size(); i++)
+			array[i] = getStringValue(col, i);
+		return array;
+	}
+
 	/** Returns a copy of the given column as a float array,
 		or null if the column is empty. */
 	public float[] getColumn(int column) {
@@ -356,7 +395,7 @@ public class ResultsTable implements Cloneable {
 		return rt;
 	}
 
-	/** Returns true if the specified column exists and is not empty. */
+	/** Returns 'true' if the specified column exists and is not empty. */
 	public boolean columnExists(int column) {
 		if ((column<0) || (column>=maxColumns))
 			return false;
@@ -546,10 +585,10 @@ public class ResultsTable implements Cloneable {
 		setValue(column, row, Double.NaN);
 		if (stringColumns==null)
 			stringColumns = new Hashtable();
-		ArrayList stringColumn = (ArrayList)stringColumns.get(new Integer(column));
+		ArrayList stringColumn = (ArrayList)stringColumns.get(Integer.valueOf(column));
 		if (stringColumn==null) {
 			stringColumn = new ArrayList();
-			stringColumns.put(new Integer(column), stringColumn);
+			stringColumns.put(Integer.valueOf(column), stringColumn);
 		}
 		int size = stringColumn.size();
 		if (row>=size) {
@@ -561,6 +600,22 @@ public class ResultsTable implements Cloneable {
 		else
 			stringColumn.set(row, value);
 	}
+	
+	/** Sets the values of the given column to the values in the array.
+	 *  If the specified column does not exist, it is created.
+	 *  When adding columns, <code>show()</code> must be called to
+	 *  update the window that displays the table.
+	 *  If the array is shorter than the column length, the remaining
+	 *  values of the column are left unchanged. If the array is longer,
+	 *  the table is extended. String values are unaffected, but only
+	 *  used if the numeric value at the given position is NaN. */
+	public void setValues(String column, double[] values) {
+		if (values.length > 0)
+			setValue(column, 0, values[0]); //creates the column if required
+		int col = getColumnIndex(column);
+		for (int i=1; i<values.length; i++)
+			setValue(col, i, values[i]);
+	}
 
 	/** Returns a tab or comma delimited string containing the column headings. */
 	public String getColumnHeadings() {
@@ -643,7 +698,7 @@ public class ResultsTable implements Cloneable {
 			if (columns[i]!=null) {
 				String value = getValueAsString(i,row);
 				if (quoteCommas) {
-					if (value.contains(","))
+					if (value!=null && value.contains(","))
 						value = "\""+value+"\"";
 				}
 				sb.append(value);
@@ -675,7 +730,7 @@ public class ResultsTable implements Cloneable {
 			double value = columns[col][row];
 			String str = null;
 			if (Double.isNaN(value) && stringColumns!=null) {
-				ArrayList stringColumn = (ArrayList)stringColumns.get(new Integer(col));
+				ArrayList stringColumn = (ArrayList)stringColumns.get(Integer.valueOf(col));
 				if (stringColumn!=null && row>=0 && row<stringColumn.size()) {
 						str = (String)stringColumn.get(row);
 						if (firstValueNumeric && "".equals(str)) {
@@ -736,7 +791,7 @@ public class ResultsTable implements Cloneable {
 		//IJ.log("getValueAsString1: col="+column+ ", row= "+row+", value= "+value+", size="+stringColumns.size());
 		if (Double.isNaN(value) && stringColumns!=null) {
 			String string = "NaN";
-			ArrayList stringColumn = (ArrayList)stringColumns.get(new Integer(column));
+			ArrayList stringColumn = (ArrayList)stringColumns.get(Integer.valueOf(column));
 			if (stringColumn==null)
 				return string;
 			//IJ.log("getValueAsString2: "+column+ +row+" "+stringColumn.size());
@@ -812,11 +867,20 @@ public class ResultsTable implements Cloneable {
 	}
 
 	public void showRowNumbers(boolean showNumbers) {
+		if (!showNumbers && this==Analyzer.getResultsTable()) {
+			IJ.beep();
+			IJ.showStatus("Standard \"Results\" table always has row numbers");
+			return;
+		}
 		showRowNumbers = showNumbers;
 		baseRowNumber = 1;
 		showRowNumbersSet = true;
 	}
 
+	public boolean showRowNumbers() {
+		return showRowNumbers;
+	}
+
 	public void showRowIndexes(boolean showIndexes) {
 		showRowNumbers = showIndexes;
 		baseRowNumber = showIndexes?0:1;
@@ -839,7 +903,7 @@ public class ResultsTable implements Cloneable {
 			return "3.4e38";
 		double np = n;
 		if (n<0.0) np = -n;
-		if ((np<0.001 && np!=0.0 && np<1.0/Math.pow(10,decimalPlaces)) || np>999999999999d || decimalPlaces<0) {
+		if ((np!=0.0 && np<1.0/Math.pow(10,decimalPlaces)) || np>999999999999d || decimalPlaces<0) {
 			if (decimalPlaces<0) {
 				decimalPlaces = -decimalPlaces;
 				if (decimalPlaces>9) decimalPlaces=9;
@@ -894,7 +958,7 @@ public class ResultsTable implements Cloneable {
 			if (columns[col]!=null) {
 				for (int i=rowIndex; i<counter-1; i++)
 					columns[col][i] = columns[col][i+1];
-				ArrayList stringColumn = stringColumns!=null?(ArrayList)stringColumns.get(new Integer(col)):null;
+				ArrayList stringColumn = stringColumns!=null?(ArrayList)stringColumns.get(Integer.valueOf(col)):null;
 				if (stringColumn!=null && stringColumn.size()==counter) {
 					for (int i=rowIndex; i<counter-1; i++)
 						stringColumn.set(i,stringColumn.get(i+1));
@@ -988,8 +1052,11 @@ public class ResultsTable implements Cloneable {
 		title = windowTitle;
 		if (!windowTitle.equals("Results") && this==Analyzer.getResultsTable())
 			IJ.log("ResultsTable.show(): the system ResultTable should only be displayed in the \"Results\" window.");
-		if (windowTitle.equals("Results") && !showRowNumbersSet)
-			showRowNumbers(true);
+		if (windowTitle.equals("Results")) {
+			if(!showRowNumbersSet)
+				showRowNumbers(true);
+			isResultsTable = true;
+		}
 		String tableHeadings = getColumnHeadings();		
 		TextPanel tp;
 		boolean newWindow = false;
@@ -1028,7 +1095,8 @@ public class ResultsTable implements Cloneable {
 				int height = 300;
 				if (size()>15) height = 400;
 				if (size()>30 && width>300) height = 500;
-				win = new TextWindow(windowTitle, "", width, height);
+				String wtitle = windowTitle + (isResultsTable&&showRowNumbers?"(Results)":"");
+				win = new TextWindow(wtitle, "", width, height);
 				cloneNeeded = true;
 			}
 			tp = win.getTextPanel();
@@ -1143,6 +1211,9 @@ public class ResultsTable implements Cloneable {
 		String[] headings = lines[0].split(cellSeparator);
 		if (headings.length<1)
 			throw new IOException("This is not a tab or comma delimited text file.");
+		String zeroWidthSpace = "\uFEFF";
+		if (headings[0].startsWith(zeroWidthSpace))
+			headings[0] = headings[0].substring(1, headings[0].length());
 		int numbersInHeadings = 0;
 		for (int i=0; i<headings.length; i++) {
 			if (headings[i].equals("NaN") || !Double.isNaN(Tools.parseDouble(headings[i])))
@@ -1193,8 +1264,9 @@ public class ResultsTable implements Cloneable {
 				if (j==labelsIndex&&labels)
 					rt.addLabel(headings[labelsIndex], items[labelsIndex]);
 				else {
-					double value = j<items.length?Tools.parseDouble(items[j]):Double.NaN;
-					if (Double.isNaN(value)) {
+					double defaultValue = -Double.MAX_VALUE;
+					double value = j<items.length?Tools.parseDouble(items[j], defaultValue):Double.NaN;
+					if (value==defaultValue) {
 						String item = j<items.length?items[j]:"";
 						if (commasReplaced) {
 							item = item.replaceAll(commaSubstitute2, ",");
@@ -1260,7 +1332,7 @@ public class ResultsTable implements Cloneable {
 	}
 
 	public boolean saveAndRename(String path) {
-		if (!"Results".equals(title))
+		if (title!=null && !title.equals("Results"))
 			renameWhenSaving = true;
 		boolean ok = save(path);
 		renameWhenSaving = false;
@@ -1318,6 +1390,7 @@ public class ResultsTable implements Cloneable {
 	public synchronized Object clone() {
 		try { 
 			ResultsTable rt2 = (ResultsTable)super.clone();
+			rt2.isResultsTable = isResultsTable;
 			rt2.headings = new String[headings.length];
 			for (int i=0; i<=lastColumn; i++)
 				rt2.headings[i] = headings[i];
@@ -1538,7 +1611,7 @@ public class ResultsTable implements Cloneable {
 		ComparableEntry[] ces = new ComparableEntry[size()];
 		ArrayList stringColumn = null;
 		if (stringColumns!=null)
-		    stringColumn = (ArrayList) stringColumns.get (new Integer (col));
+		    stringColumn = (ArrayList) stringColumns.get (Integer.valueOf(col));
 		for (int i = 0; i < size(); i++) {
 		    ComparableEntry ce = new ComparableEntry();
 		    ce.index = i;
@@ -1558,9 +1631,9 @@ public class ResultsTable implements Cloneable {
 		    ArrayList sc = null;
 		    Map scs =  stringColumns;
 		    	if (scs != null)
-			sc = (ArrayList) scs.get (new Integer (i));
+			sc = (ArrayList) scs.get (Integer.valueOf(i));
 		    if (sc != null) {
-				ArrayList sc2 = (ArrayList) rt2.stringColumns.get (new Integer (i));
+				ArrayList sc2 = (ArrayList) rt2.stringColumns.get (Integer.valueOf(i));
 				for (int j = 0; j < size(); j++)
 			    	sc.set (j, sc2.get (ces[j].index));
 		    }
@@ -1590,5 +1663,9 @@ public class ResultsTable implements Cloneable {
 			return  (dValue < e.dValue) ? -1 : ( (dValue > e.dValue) ? 1 : 0 );
 		}
 	}
+	
+	public void setIsResultsTable(boolean isResultsTable) {
+		this.isResultsTable = isResultsTable;
+	}
 		
 }
