@@ -355,6 +355,24 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 		return open(path, true);
 	}
 
+	/** Opens an AVI file, where 'options' can contain
+	 * 'virtual' (open as virtual stack),
+	 * 'convert' (convert color images to grayscale) or
+	 * 'flip' (flip vertically).
+	 * The ImagePlus is not displayed.
+	*/
+	public static ImagePlus open(String path, String options) {
+		AVI_Reader reader = new AVI_Reader();
+		boolean virtual = options.contains("virtual");
+		boolean convertToGray = options.contains("convert");
+		boolean flipVertical = options.contains("flip");
+		ImageStack stack = reader.makeStack (path, 1, 0, virtual, convertToGray, flipVertical);
+		if (stack!=null)
+			return new ImagePlus((new File(path)).getName(), stack);
+		else
+			return null;
+	}
+
 	/** Opens an AVI file as a stack in memory or a virtual stack. The ImagePlus is not displayed. */
 	public static ImagePlus open(String path, boolean virtual) {
 		AVI_Reader reader = new AVI_Reader();
@@ -412,7 +430,7 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 	}
 
 	/** Returns an ImageProcessor for the specified slice of this virtual stack (if it is one)
-     *  where 1<=n<=nslices. Returns null if no virtual stack or no slices or error reading the frame.
+     *  where {@literal 1<=n<=nslices}. Returns null if no virtual stack or no slices or error reading the frame.
 	 */
 	public synchronized ImageProcessor getProcessor(int n) {
 		if (frameInfos==null || frameInfos.size()==0 || raFilePath==null)
@@ -433,12 +451,16 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 			closeFile(rFile);
 		}
 		if (pixels == null) return null; //failed
+		ImageProcessor ip = null;
 		if (pixels instanceof byte[])
-			return new ByteProcessor(dwWidth, biHeight, (byte[])pixels, cm);
+			ip = new ByteProcessor(dwWidth, biHeight, (byte[])pixels, cm);
 		else if (pixels instanceof short[])
-			return new ShortProcessor(dwWidth, biHeight, (short[])pixels, cm);
+			ip = new ShortProcessor(dwWidth, biHeight, (short[])pixels, cm);
 		else
-			return new ColorProcessor(dwWidth, biHeight, (int[])pixels);
+			ip = new ColorProcessor(dwWidth, biHeight, (int[])pixels);
+		if (ip!=null)
+			ip.setSliceNumber(n);
+		return ip;
 	}
 
 	/** Returns the image width of the virtual stack */
@@ -465,7 +487,7 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 	}
 
 	/** Deletes the specified image from this virtual stack (if it is one),
-	 *	where 1<=n<=nslices. */
+	 *	where {@literal 1<=n<=nslices}. */
 	public void deleteSlice(int n) {
 		if (frameInfos==null || frameInfos.size()==0) return;
 		if (n<1 || n>frameInfos.size())
@@ -486,21 +508,25 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 		if  (options!=null) {  //macro
 			if (options.contains("open="))
 				Macro.setOptions(options.replace("open=", "avi="));
-			int first = (int)Tools.getNumberFromList(options, "first=");
-			if (first>0) firstFrame = first;
-			int last = (int)Tools.getNumberFromList(options, "last=");
-			if (last>0) lastFrame = last;
 		}
 		if (path==null || path.length()==0)
 			path = Prefs.get(PATH_KEY, IJ.getDir("downloads")+"movie.avi");
 		GenericDialog gd = new GenericDialog("AVI Reader");
 		gd.setInsets(5, 0, 0);
-		gd.addFileField("AVI:", path);
+		gd.addFileField("AVI:", path, 29);
 		gd.setInsets(2, 40, 5);
 		gd.addMessage("drag and drop target", IJ.font10, Color.darkGray);
 		gd.addCheckbox("Use Virtual Stack", isVirtual);
 		gd.addCheckbox("Convert to Grayscale", convertToGray);
 		gd.addCheckbox("Flip Vertical", flipVertical);
+		gd.setInsets(15, 0, 3);
+		gd.addNumericField("First:", firstFrame, 0);
+		gd.addNumericField("Last:", lastFrame, lastFrame, 6, "*");
+		TextField lastField = (TextField)(gd.getNumericFields().lastElement());
+		if (lastFrame == 0)
+			lastField.setText("");
+		gd.setInsets(0, 40, 5);
+		gd.addMessage("* Leave empty or set to 0 for reading to the end.\n   Also accepts e.g. -5 to skip last 5 frames.", IJ.font10, Color.darkGray);
 		gd.showDialog();
 		if (gd.wasCanceled())
 			return false;
@@ -513,6 +539,12 @@ public class AVI_Reader extends VirtualStack implements PlugIn {
 		isVirtual = gd.getNextBoolean();
 		convertToGray = gd.getNextBoolean();
 		flipVertical = gd.getNextBoolean();
+		double first = gd.getNextNumber();
+		if (!Double.isNaN(first)) firstFrame = (int)first;
+		if (lastField.getText().length()==0)
+			lastField.setText("0");
+		double last = gd.getNextNumber();
+		if (!Double.isNaN(last)) lastFrame = (int)last;
 		if (!IJ.isMacro()) {
 			staticConvertToGray = convertToGray;
 			staticFlipVertical = flipVertical;
