@@ -110,7 +110,10 @@ public class DICOM extends ImagePlus implements PlugIn {
 		if (fi!=null && fi.width>0 && fi.height>0 && fi.offset>0) {
 			FileOpener fo = new FileOpener(fi);
 			ImagePlus imp = fo.openImage();
-			boolean openAsFloat = (dd.rescaleSlope!=1.0&&!Prefs.ignoreRescaleSlope) || Prefs.openDicomsAsFloat;		
+			// Avoid opening as float even if slope != 1.0 in case ignoreRescaleSlope or fixedDicomScaling
+			// were checked in the DICOM preferences.
+			boolean openAsFloat = (dd.rescaleSlope!=1.0 && !(Prefs.ignoreRescaleSlope || Prefs.fixedDicomScaling)) 
+				|| Prefs.openDicomsAsFloat;
 			String options = Macro.getOptions();
 			if (openAsFloat) {
 				IJ.run(imp, "32-bit", "");
@@ -124,13 +127,14 @@ public class DICOM extends ImagePlus implements PlugIn {
 					imp.setDisplayRange(stats.min,stats.max);
 				}
 			} else if (fi.fileType==FileInfo.GRAY16_SIGNED) {
-				if (dd.rescaleIntercept!=0.0 && dd.rescaleSlope==1.0) {
+				if (dd.rescaleIntercept!=0.0 && (dd.rescaleSlope==1.0||Prefs.fixedDicomScaling)) {
 					double[] coeff = new double[2];
-					coeff[0] = -32768 + dd.rescaleIntercept;
-					coeff[1] = 1.0;
+					coeff[0] = dd.rescaleSlope*(-32768) + dd.rescaleIntercept;
+					coeff[1] = dd.rescaleSlope;
 					imp.getCalibration().setFunction(Calibration.STRAIGHT_LINE, coeff, "Gray Value");
 				}
-			} else if (dd.rescaleIntercept!=0.0 && (dd.rescaleSlope==1.0||fi.fileType==FileInfo.GRAY8)) {
+			} else if (dd.rescaleIntercept!=0.0 && 
+					  (dd.rescaleSlope==1.0||Prefs.fixedDicomScaling||fi.fileType==FileInfo.GRAY8)) {
 				double[] coeff = new double[2];
 				coeff[0] = dd.rescaleIntercept;
 				coeff[1] = dd.rescaleSlope;
@@ -258,7 +262,9 @@ class DicomDecoder {
 	private static final int AE=0x4145, AS=0x4153, AT=0x4154, CS=0x4353, DA=0x4441, DS=0x4453, DT=0x4454,
 		FD=0x4644, FL=0x464C, IS=0x4953, LO=0x4C4F, LT=0x4C54, PN=0x504E, SH=0x5348, SL=0x534C, 
 		SS=0x5353, ST=0x5354, TM=0x544D, UI=0x5549, UL=0x554C, US=0x5553, UT=0x5554,
-		OB=0x4F42, OW=0x4F57, SQ=0x5351, UN=0x554E, QQ=0x3F3F;
+		OB=0x4F42, OW=0x4F57, SQ=0x5351, UN=0x554E, QQ=0x3F3F,
+		OF=0x4F46, OL=0x4F4C, OD=0x4F44, UC=0x5543, UR=0x5552, OV=0x4F56, SV=0x5356, UV=0x5556;
+		
 		
 	private static Properties dictionary;
 
@@ -461,6 +467,8 @@ class DicomDecoder {
 		
 		switch (vr) {
 			case OB: case OW: case SQ: case UN: case UT:
+			case OF: case OL: case OD: case UC: case UR: 
+			case OV: case SV: case UV:
 				// Explicit VR with 32-bit length if other two bytes are zero
 				if ( (b2 == 0) || (b3 == 0) ) return getInt();
 				// Implicit VR with 32-bit length
@@ -468,10 +476,10 @@ class DicomDecoder {
 				if (littleEndian)
 					return ((b3<<24) + (b2<<16) + (b1<<8) + b0);
 				else
-					return ((b0<<24) + (b1<<16) + (b2<<8) + b3);		
+					return ((b0<<24) + (b1<<16) + (b2<<8) + b3);
 			case AE: case AS: case AT: case CS: case DA: case DS: case DT:  case FD:
 			case FL: case IS: case LO: case LT: case PN: case SH: case SL: case SS:
-			case ST: case TM:case UI: case UL: case US: case QQ:
+			case ST: case TM: case UI: case UL: case US: case QQ:
 				// Explicit vr with 16-bit length
 				if (littleEndian)
 					return ((b3<<8) + b2);
@@ -939,7 +947,7 @@ class DicomDecoder {
  		if (s.startsWith("\\"))
  			s = s.substring(1);
 		Double d;
-		try {d = new Double(s);}
+		try {d = Double.valueOf(s);}
 		catch (NumberFormatException e) {d = null;}
 		if (d!=null)
 			return(d.doubleValue());
@@ -1184,7 +1192,7 @@ class DicomDictionary {
 		"00181147=CSField of View Shape",
 		"00181149=ISField of View Dimensions(s)",
 		"00181150=ISExposure Time",
-		"00181151=ISX-ray Tube Current",
+		"00181151=ISX-Ray Tube Current",
 		"00181152=ISExposure",
 		"00181153=ISExposure in uAs",
 		"00181154=DSAverage Pulse Width",
@@ -1350,6 +1358,8 @@ class DicomDictionary {
 		"00187062=LTExposure Control Mode Description",
 		"00187064=CSExposure Status",
 		"00187065=DSPhototimer Setting",
+		"00188151=DSX-Ray Tube Current in µA",
+		"00189330=DSX-Ray Tube Current in mA",
 		
 		"0020000D=UIStudy Instance UID",
 		"0020000E=UISeries Instance UID",
@@ -1717,4 +1727,3 @@ class DicomDictionary {
 	};
 
 }
-
