@@ -14,14 +14,14 @@ import ij.plugin.frame.Recorder;
 
 /** Implements the File/Import/Image Sequence command, which
 	opens a folder of images as a stack. */
-public class FolderOpener implements PlugIn {
-	private static final int MAX_SEPARATE = 40;
+public class FolderOpener implements PlugIn, TextListener {
+	private static final int MAX_SEPARATE = 100;
 	private static final String DIR_KEY = "import.sequence.dir";
 	private static final String[] types = {"default", "16-bit", "32-bit", "RGB"};
-	private static String[] excludedTypes = {".txt", ".lut", ".roi", ".pty", ".hdr", ".java", ".ijm", ".py", ".js", ".bsh", ".xml"};
+	private static String[] excludedTypes = {".txt",".lut",".roi",".pty",".hdr",".java",".ijm",".py",".js",".bsh",".xml",".rar",".h5",".doc",".xls"};
 	private static boolean staticSortFileNames = true;
 	private static boolean staticOpenAsVirtualStack;
-	private boolean convertToRGB;
+	private boolean convertToGrayscale;  //unused
 	private boolean sortFileNames = true;
 	private boolean sortByMetaData = true;
 	private boolean openAsVirtualStack;
@@ -42,6 +42,8 @@ public class FolderOpener implements PlugIn {
 	private int step = 1;
 	private double scale = 100.0;
 	private boolean openAsSeparateImages;
+	private boolean runningOpen;	
+	private TextField dirField, filterField, startField, countField, stepField;
 
 	
 	/** Opens the images in the specified directory as a stack. Displays
@@ -52,11 +54,14 @@ public class FolderOpener implements PlugIn {
 
 	/** Opens the images in the specified directory as a stack. Opens
 		the images as a virtual stack if the 'options' string contains
-		'virtual' or 'use'. Add ' file=abc' to the options string to only open
-		images with, for example, 'abc' in their name. Add ' noMetaSort' to
-		disable sorting of DICOM stacks by series number (0020,0011).
-		Displays directory chooser and options dialogs if the the 'path'
-		argument is null. */
+		'virtual' or 'use'. Add ' filter=abc' to the options string to only open
+		images with, for example, 'abc' in their name. The image type, start,
+		step, count and scale can also be set, for example
+		"bitdepth=32 start=10 step=2 count=10 scale=50".
+		Add ' noMetaSort' to disable sorting of DICOM stacks by series
+		number (0020,0011). Displays the Import/Sequence dialog if the
+		the 'path' argument is null.
+	*/
 	public static ImagePlus open(String path, String options) {
 		FolderOpener fo = new FolderOpener();
 		fo.saveImage = true;
@@ -67,9 +72,10 @@ public class FolderOpener implements PlugIn {
 
 	/** Opens the images in the specified directory as a widthxheight stack.
  		Opens the images as a virtual stack if the 'options' string contains
-		'virtual' or 'use'. Add ' file=abc' to the options string to only open
-		images with, for example, 'abc' in their name. Add ' noMetaSort' to
-		disable sorting of DICOM stacks by series number (0020,0011).
+		'virtual'. Add ' filter=abc' to the options string to only open
+		images with, for example, 'abc' in their name. The image type, start,
+		step, count and scale can also be set, for example
+		"bitdepth=32 start=10 step=2 count=10 scale=50".
 	*/
 	public static ImagePlus open(String path, int width, int height, String options) {
 		FolderOpener fo = new FolderOpener();
@@ -92,6 +98,13 @@ public class FolderOpener implements PlugIn {
 		this.start = (int)Tools.getNumberFromList(options,"start=",1);
 		this.step = (int)Tools.getNumberFromList(options,"step=",1);
 		this.scale = Tools.getNumberFromList(options,"scale=",100);
+		this.nFiles = (int)Tools.getNumberFromList(options,"count=",0);
+		if (options.contains(" open")) {
+			this.openAsSeparateImages = true;
+			this.openAsVirtualStack = true;
+			this.saveImage = false;
+			this.runningOpen = true;
+		}
 	}
 
 	/** Opens the images in the specified directory as a stack. Displays
@@ -106,24 +119,16 @@ public class FolderOpener implements PlugIn {
 		boolean isMacro = Macro.getOptions()!=null;
 		if (!directorySet)
 			directory = null;
-		if (arg!=null && !arg.equals("")) {
+		if (arg!=null && !arg.equals(""))
 			directory = arg;
-		} else {
+		else {
 			if (!isMacro) {
 				sortFileNames = staticSortFileNames;
 				openAsVirtualStack = staticOpenAsVirtualStack;
 			}
 			arg = null;
-			String title = "Open Image Sequence...";
 			String macroOptions = Macro.getOptions();
 			if (macroOptions!=null) {
-				directory = Macro.getValue(macroOptions, title, null);
-				if (directory!=null) {
-					directory = OpenDialog.lookupPathVariable(directory);
-					File f = new File(directory);
-					if (!f.isDirectory() && (f.exists()||directory.lastIndexOf(".")>directory.length()-5))
-						directory = f.getParent();
-				}
 				legacyRegex = Macro.getValue(macroOptions, "or", "");
 				if (legacyRegex.equals(""))
 					legacyRegex = null;
@@ -132,19 +137,29 @@ public class FolderOpener implements PlugIn {
 		if (arg==null) {
 			if (!showDialog()) return;
 		}
+		if (directory==null || directory.length()==0) {
+			error("No directory specified.     ");
+			return;
+		}
 		File file = new File(directory);
 		String[] list = file.list();
 		if (list==null) {
 			String parent = file.getParent();
-			file = new File(parent);
-			list = file.list();
+			if (parent!=null) {
+				file = new File(parent);
+				list = file.list();
+			}
 			if (list!=null)
 				directory = parent;
 			else {
-				IJ.error("File>Import>Image Sequence", "Directory not found: "+directory);
+				error("Directory not found: "+directory);
 				return;
 			}
 		}
+		if (!(directory.endsWith("/")||directory.endsWith("\\")))
+			directory += "/";
+		if (arg==null && !isMacro)
+			Prefs.set(DIR_KEY, directory);
 		//remove subdirectories from list
 		ArrayList fileList = new ArrayList();
 		for (int i=0; i<list.length; i++) {
@@ -156,21 +171,18 @@ public class FolderOpener implements PlugIn {
 			list = (String[])fileList.toArray(new String[fileList.size()]);
 
 		String title = directory;
-		if (title.endsWith(File.separator) || title.endsWith("/"))
-			title = title.substring(0, title.length()-1);
-		int index = title.lastIndexOf(File.separatorChar);
-		if (index!=-1)
-			title = title.substring(index + 1);
-		else {
-			index = title.lastIndexOf("/");
-			if (index!=-1)
-				title = title.substring(index + 1);
-		}
-		if (title.endsWith(":"))
-			title = title.substring(0, title.length()-1);
-		
+		title = trimTitle(title);	
 		list = trimFileList(list);
-		if (list==null) return;
+		if (list==null)
+			return;
+		String pluginName = "Sequence Reader";
+		if (legacyRegex!=null)
+			pluginName += "(legacy)";
+		list = getFilteredList(list, filter, pluginName);
+		if (list==null)
+			return;
+		if (sortFileNames || IJ.isMacOSX())
+			list = StringSorter.sortNumerically(list);
 		if (IJ.debugMode) IJ.log("FolderOpener: "+directory+" ("+list.length+" files)");
 		int width=0, height=0, stackSize=1;
 		ImageStack stack = null;
@@ -188,7 +200,7 @@ public class FolderOpener implements PlugIn {
 				Opener opener = new Opener();
 				opener.setSilentMode(true);
 				IJ.redirectErrorMessages(true);
-				ImagePlus imp = opener.openImage(directory, list[i]);
+				ImagePlus imp = opener.openTempImage(directory, list[i]);
 				IJ.redirectErrorMessages(false);
 				if (imp!=null) {
 					width = imp.getWidth();
@@ -204,20 +216,14 @@ public class FolderOpener implements PlugIn {
 				}
 			}
 			if (width==0) {
-				IJ.error("Sequence Reader", "This folder does not appear to contain\n"
+				error("This folder does not appear to contain\n"
 				+ "any TIFF, JPEG, BMP, DICOM, GIF, FITS or PGM files.\n \n"
 				+ "   \""+directory+"\"");
 				return;
 			}
-			String pluginName = "Sequence Reader";
-			if (legacyRegex!=null)
-				pluginName += "(legacy)";
-			list = getFilteredList(list, filter, pluginName);
-			if (list==null)
-				return;
 			IJ.showStatus("");
 			t0 = System.currentTimeMillis();
-			if (sortFileNames || dicomImages || IJ.isMacOSX())
+			if (dicomImages && !IJ.isMacOSX() && !sortFileNames)
 				list = StringSorter.sortNumerically(list);
 
 			if (this.nFiles<1)
@@ -243,7 +249,7 @@ public class FolderOpener implements PlugIn {
 					IJ.open(directory+list[i]);
 					imp = null;
 				} else if (!openAsVirtualStack||stack==null) {
-					imp = opener.openImage(directory, list[i]);
+					imp = opener.openTempImage(directory, list[i]);
 					stackSize = imp!=null?imp.getStackSize():1;
 				}
 				IJ.redirectErrorMessages(false);
@@ -254,13 +260,13 @@ public class FolderOpener implements PlugIn {
 						width = stackWidth;
 						height = stackHeight;
 					}
-					if (bitDepth==0) bitDepth = imp.getBitDepth();
+					if (bitDepth==0)
+						bitDepth = imp.getBitDepth();
 					fi = imp.getOriginalFileInfo();
 					ImageProcessor ip = imp.getProcessor();
 					min = ip.getMin();
 					max = ip.getMax();
 					cal = imp.getCalibration();
-					if (convertToRGB) bitDepth = 24;
 					ColorModel cm = imp.getProcessor().getColorModel();
 					if (openAsVirtualStack) {
 						if (stackSize>1) {
@@ -340,10 +346,6 @@ public class FolderOpener implements PlugIn {
 								label2 += ":"+slice;
 						}
 						ip = inputStack.getProcessor(slice);
-						if (convertToRGB) {
-							ip = ip.convertToRGB();
-							bitDepth2 = 24;
-						}
 						if (bitDepth2!=bitDepth) {
 							if (dicomImages && bitDepth==16 && bitDepth2==32 && this.scale==100) {
 								ip = ip.convertToFloat();
@@ -421,15 +423,24 @@ public class FolderOpener implements PlugIn {
 				}
 			}
 			if (imp2.getStackSize()==1) {
-				imp2.setProperty("Label", list[0]);
+				int idx = this.start-1;
+				if (idx<0 || idx>=list.length)
+					idx = 0;
+				imp2.setProp("Slice_Label", list[idx]);
 				if (info1!=null)
 					imp2.setProperty("Info", info1);
 			}
-			if (arg==null && !saveImage) {
+			if ((arg==null||runningOpen) && !saveImage) {
 				String time = (System.currentTimeMillis()-t0)/1000.0 + " seconds";
-				if (openAsSeparateImages && imp2.getStackSize()<=MAX_SEPARATE)
+				if (openAsSeparateImages) {
+					if (imp2.getStackSize()>MAX_SEPARATE && !IJ.isMacro()) {
+						boolean ok = IJ.showMessageWithCancel("Import>Image Sequence",
+						"Are you sure you want to open "+imp2.getStackSize()
+						+" separate windows?\nThis may cause the system to become very slow or stall.");
+						if (!ok) return;
+					}
 					openAsSeparateImages(imp2);
-				else
+				} else
 					imp2.show(time);
 				if (stack.isVirtual()) {
 					overlay = stack.getProcessor(1).getOverlay();
@@ -439,12 +450,10 @@ public class FolderOpener implements PlugIn {
 			}
 			if (saveImage)
 				image = imp2;
-			if (openAsSeparateImages && imp2.getStackSize()>MAX_SEPARATE)
-				IJ.error("Import>Image Sequence", "A maximum of "+MAX_SEPARATE+" images can be opened separately.");
 		}
 		IJ.showProgress(1.0);
 		if (Recorder.record) {
-			String options = openAsVirtualStack?"virtual":"";
+			String options = openAsVirtualStack&&!openAsSeparateImages?"virtual":"";
 			if (bitDepth!=defaultBitDepth)
 				options = options + " bitdepth=" + bitDepth;				
 			if (filter!=null && filter.length()>0) {
@@ -460,18 +469,55 @@ public class FolderOpener implements PlugIn {
 				options = options + " scale=" + scale;				
 			if (!sortByMetaData)
 				options = options + " noMetaSort";
+			if (!Recorder.scriptMode() && openAsSeparateImages)
+				options = options + " open";
 			String dir = Recorder.fixPath(directory);
-   			Recorder.recordCall("imp = FolderOpener.open(\""+dir+"\", \""+options+"\");");
+			if (Recorder.scriptMode())
+   				Recorder.recordCall("imp = FolderOpener.open(\""+dir+"\", \""+options+"\");");
+   			else {
+   				if (options.length()==0)
+   					Recorder.recordString("File.openSequence(\""+dir+"\");\n");
+   				else
+   					Recorder.recordString("File.openSequence(\""+dir+"\", \""+options+"\");\n");
+   				Recorder.disableCommandRecording();
+   			}
 		}
 	}
 	
+	public static String trimTitle(String title) {
+		if (title.endsWith(File.separator) || title.endsWith("/"))
+			title = title.substring(0, title.length()-1);
+		int index = title.lastIndexOf(File.separatorChar);
+		if (index!=-1)
+			title = title.substring(index + 1);
+		else {
+			index = title.lastIndexOf("/");
+			if (index!=-1)
+				title = title.substring(index + 1);
+		}
+		if (title.endsWith(":"))
+			title = title.substring(0, title.length()-1);
+		return title;
+	}
+
+	private void error(String msg) {
+		IJ.error("Import>Image Sequence", msg);
+	}
+	
 	private void openAsSeparateImages(ImagePlus imp) {
 		VirtualStack stack = (VirtualStack)imp.getStack();
 		String dir = stack.getDirectory();
-		for (int n=1; n<=stack.size(); n++)
-			IJ.open(dir+stack.getFileName(n));	
+		int skip = 0;
+		for (int n=1; n<=stack.size(); n++) {
+			ImagePlus imp2 = IJ.openImage(dir+stack.getFileName(n));
+			if (skip<=0) {
+				imp2.show();
+				skip = imp2.getStackSize()-1;
+			} else
+				skip--;
+		}	
 	}
-	
+
 	public static boolean useInfo(String info) {
 		return info!=null && !(info.startsWith("Software")||info.startsWith("ImageDescription"));
 	 }
@@ -518,13 +564,14 @@ public class FolderOpener implements PlugIn {
 				this.bitDepth = 24;
 		}
 		String countStr = "---";
-		if (directorySet) {
+		if (!directorySet && options==null)
+			directory = Prefs.get(DIR_KEY, IJ.getDir("downloads")+"stack/");
+		if (directory!=null && !IJ.isMacro()) {			
 			File f = new File(directory);
 			String[] names = f.list();
-			names = (new FolderOpener()).trimFileList(names);
-			countStr = ""+names.length;
-		} else
-			directory = Prefs.get(DIR_KEY, IJ.getDir("downloads")+"stack/");
+			names = trimFileList(names);
+			countStr = names!=null?""+names.length:"---";
+		}
 		GenericDialog gd = new GenericDialog("Import Image Sequence");
 		gd.setInsets(5, 0, 0);
 		gd.addDirectoryField("Dir:", directory);		
@@ -537,16 +584,28 @@ public class FolderOpener implements PlugIn {
 		gd.addNumericField("Start:", this.start, 0, 6, "");
 		gd.addStringField("Count:", countStr, 6);
 		gd.addNumericField("Step:", this.step, 0, 6, "");
+		if (!IJ.isMacro() && !GraphicsEnvironment.isHeadless()) {
+			Vector v = gd.getStringFields();
+			dirField = (TextField)v.elementAt(0);
+			dirField.addTextListener(this);
+			filterField = (TextField)v.elementAt(1);
+			filterField.addTextListener(this);
+			countField = (TextField)v.elementAt(2);
+			v = gd.getNumericFields();
+			startField = (TextField)v.elementAt(0);
+			startField.addTextListener(this);
+			stepField = (TextField)v.elementAt(1);
+			stepField.addTextListener(this);
+		}
 		gd.addNumericField("Scale:", this.scale, 0, 6, "%");
 		gd.addCheckbox("Sort names numerically", sortFileNames);
 		gd.addCheckbox("Use virtual stack", openAsVirtualStack);
 		gd.addCheckbox("Open as separate images", false);		
-		gd.addHelp(IJ.URL+"/docs/menus/file.html#seq1");
+		gd.addHelp(IJ.URL2+"/docs/menus/file.html#seq1");
 		gd.showDialog();
 		if (gd.wasCanceled())
 			return false;
 		directory = gd.getNextString();
-		Prefs.set(DIR_KEY, directory);
 		gd.setSmartRecording(true);
 		int index = gd.getNextChoiceIndex();
 		bitDepth = typeToBitDepth(types[index]);
@@ -665,6 +724,8 @@ public class FolderOpener implements PlugIn {
 
 	/** Removes names that start with "." or end with ".db", ".txt", ".lut", "roi", ".pty", ".hdr", ".py", etc. */
 	public String[] trimFileList(String[] rawlist) {
+		if (rawlist==null)
+			return null;
 		int count = 0;
 		for (int i=0; i< rawlist.length; i++) {
 			String name = rawlist[i];
@@ -708,6 +769,7 @@ public class FolderOpener implements PlugIn {
 		sortByMetaData = b;
 	}
 
+	/** Used by DragAndDrop when a directory is dragged onto the ImageJ window. */
 	public void setDirectory(String path) {
 		directory = path;
 		directorySet = true;
@@ -720,6 +782,25 @@ public class FolderOpener implements PlugIn {
 	public String[] sortFileList(String[] list) {
 		return StringSorter.sortNumerically(list);
 	}
+	
+	public void textValueChanged(TextEvent e) {
+		if (dirField==null)
+			return;
+		String dir = dirField.getText();
+		File f = new File(dir);
+		String[] names = f.list();
+		names = trimFileList(names);
+		names = getFilteredList(names, filterField.getText(), null);
+		int count = names!=null?names.length:0;
+		double start = Tools.parseDouble(startField.getText(), Double.NaN);
+		if (!Double.isNaN(start) && start>1)
+			count = count - ((int)start-1);
+		double step = Tools.parseDouble(stepField.getText(), Double.NaN);
+		if (!Double.isNaN(step) && step>1)
+			count = count/(int)step;
+		String countStr = count>0?""+count:"---";
+		countField.setText(countStr);
+	}
 
 } // FolderOpener
 
