@@ -60,13 +60,14 @@ public class ImageInfo implements PlugIn {
 	public String getImageInfo(ImagePlus imp) {
 		ImageProcessor ip = imp.getProcessor();
 		String infoProperty = null;
-		if (imp.getStackSize()>1) {
+		boolean isStack = imp.getStackSize()>1;
+		if (isStack || imp.hasImageStack()) {
 			ImageStack stack = imp.getStack();
 			String label = stack.getSliceLabel(imp.getCurrentSlice());
 			if (label!=null && label.indexOf('\n')>0)
 				infoProperty = label;
 		}
-		if (infoProperty==null) {
+		if (infoProperty==null || (isStack && (imp.getStack() instanceof ListVirtualStack))) {
 			infoProperty = (String)imp.getProperty("Info");
 			if (infoProperty==null)
 				infoProperty = getExifData(imp);
@@ -87,7 +88,7 @@ public class ImageInfo implements PlugIn {
 			return info;
 	}
 
-	private String getExifData(ImagePlus imp) {
+	public String getExifData(ImagePlus imp) {
 		FileInfo fi = imp.getOriginalFileInfo();
 		if (fi==null)
 			return null;
@@ -188,37 +189,53 @@ public class ImageInfo implements PlugIn {
     	switch (type) {
 	    	case ImagePlus.GRAY8:
 	    		s += "Bits per pixel: 8 ";
-	    		String lut = "LUT";
-	    		if (imp.getProcessor().isColorLut())
-	    			lut = "color " + lut;
-	    		else
-	    			lut = "grayscale " + lut;
-	    		if (imp.isInvertedLut())
-	    			lut = "inverting " + lut;
+	    		String lut = getLutInfo(imp);
 				s += "(" + lut + ")\n";
 				if (imp.getNChannels()>1)
 					s += displayRanges(imp);
-				else
+				else {
 					s += "Display range: "+(int)ip.getMin()+"-"+(int)ip.getMax()+"\n";
+					ip.resetRoi();
+					ImageStatistics stats = ip.getStats();
+					s += "Pixel value range: "+(int)stats.min+"-"+(int)stats.max+"\n";
+				}
 				break;
 	    	case ImagePlus.GRAY16: case ImagePlus.GRAY32:
 	    		if (type==ImagePlus.GRAY16) {
-	    			String sign = cal.isSigned16Bit()?"signed":"unsigned";
-	    			s += "Bits per pixel: 16 ("+sign+")\n";
+	    			String sign = cal.isSigned16Bit()?"signed, ":"unsigned, ";
+	    			s += "Bits per pixel: 16 ("+sign+getLutInfo(imp)+")\n";
 	    		} else
-	    			s += "Bits per pixel: 32 (float)\n";
+	    			s += "Bits per pixel: 32 (float, "+getLutInfo(imp)+")\n";
 				if (imp.getNChannels()>1)
 					s += displayRanges(imp);
 				else {
+					String pvrLabel = "Pixel value range: ";
 					s += "Display range: ";
 					double min = ip.getMin();
 					double max = ip.getMax();
-					if (cal.calibrated()) {
+					String dash = "-";
+					if (type==ImagePlus.GRAY32||cal.calibrated())
+						dash = " - ";
+					if (type==ImagePlus.GRAY32)
+						s += d2s(min) + dash + d2s(max) + "\n";
+					else if (cal.calibrated()) {
+						pvrLabel = "Raw pixel value range: ";
 						min = cal.getCValue((int)min);
 						max = cal.getCValue((int)max);
+						s += d2s(min) + dash + d2s(max) + "\n";
+					} else
+						s += (int)min+dash+(int)max+"\n";
+					ip.resetRoi();
+					ImageStatistics stats = ip.getStats();
+					if (ip.isSigned16Bit()) {
+						stats.min -= 32768;
+						stats.max -= 32768;
 					}
-					s += d2s(min) + " - " + d2s(max) + "\n";
-				}
+					if (type==ImagePlus.GRAY32)
+						s += pvrLabel+d2s(stats.min)+dash+d2s(stats.max)+"\n";
+					else
+						s += pvrLabel+(int)stats.min+dash+(int)stats.max+"\n";
+			}
 				break;
 	    	case ImagePlus.COLOR_256:
 	    		s += "Bits per pixel: 8 (color LUT)\n";
@@ -243,13 +260,12 @@ public class ImageInfo implements PlugIn {
     			label = " (" + label + ")";
     		else
     			label = "";
-			if (interval>0.0 || fps!=0.0) {
+			if (imp.getNFrames()>1 || interval>0.0 || fps!=0.0) {
 				s += "Frame: " + number + label + "\n";
 				if (fps!=0.0) {
 					String sRate = Math.abs(fps-Math.round(fps))<0.00001?IJ.d2s(fps,0):IJ.d2s(fps,5);
 					s += "Frame rate: " + sRate + " fps\n";
-				}
-				if (interval!=0.0)
+				} else
 					s += "Frame interval: " + ((int)interval==interval?IJ.d2s(interval,0):IJ.d2s(interval,5)) + " " + cal.getTimeUnit() + "\n";
 			} else
 				s += "Image: " + number + label + "\n";
@@ -277,11 +293,15 @@ public class ImageInfo implements PlugIn {
 					stackType += " (ListVirtualStack)";
 				s += "Stack type: " + stackType+ "\n";
 			}
+		} else if (imp.hasImageStack()) { // one image stack
+    		String label = imp.getStack().getShortSliceLabel(1);
+    		if (label!=null && label.length()>0)
+				s += "Image: 1/1 (" + label + ")\n";
 		}
 
 		if (imp.isLocked())
 			s += "**Locked**\n";
-		if (ip.getMinThreshold()==ImageProcessor.NO_THRESHOLD)
+		if (!ip.isThreshold())
 			s += "No threshold\n";
 	    else {
 	    	double lower = ip.getMinThreshold();
@@ -391,7 +411,7 @@ public class ImageInfo implements PlugIn {
 			s += "Macro is running"+(Interpreter.isBatchMode()?" in batch mode":"")+"\n";
 
 	    Roi roi = imp.getRoi();
-	    if (roi == null) {
+	    if (roi==null) {
 			if (cal.calibrated())
 	    		s += " \n";
 	    	s += "No selection\n";
@@ -462,6 +482,17 @@ public class ImageInfo implements PlugIn {
 
 		return s;
 	}
+	
+	private String getLutInfo(ImagePlus imp) {
+		String lut = "LUT";
+		if (imp.getProcessor().isColorLut())
+			lut = "color " + lut;
+		else
+			lut = "grayscale " + lut;
+		if (imp.isInvertedLut())
+			lut = "inverting " + lut;
+		return lut;
+	}
 
 	private String displayRanges(ImagePlus imp) {
 		LUT[] luts = imp.getLuts();
@@ -489,7 +520,10 @@ public class ImageInfo implements PlugIn {
 	}
 
 	private void showInfo(ImagePlus imp, String info, int width, int height) {
-		new TextWindow("Info for "+imp.getTitle(), info, width, height);
+		if (GraphicsEnvironment.isHeadless())
+			IJ.log(info);
+		else
+			new TextWindow("Info for "+imp.getTitle(), info, width, height);
 		//Editor ed = new Editor();
 		//ed.setSize(width, height);
 		//ed.create("Info for "+imp.getTitle(), info);
@@ -509,7 +543,7 @@ public class ImageInfo implements PlugIn {
 			String value = props[i+1];
 			if (LUT.nameKey.equals(key) || "UniqueName".equals(key))
 				continue;
-			if (key!=null && value!=null && !key.equals("ShowInfo")) {
+			if (key!=null && value!=null && !(key.equals("ShowInfo")||key.equals("Slice_Label"))) {
 				if (value.length()<80)
 					s += key + ": " + value + "\n";
 				else
