@@ -23,8 +23,8 @@ public class Selection implements PlugIn, Measurements {
 	private static int lineWidth = 1;
 	private static boolean smooth;
 	private static boolean adjust;
-
-	
+	private static double translateX;
+	private static double translateY;
 
 	public void run(String arg) {
 		imp = WindowManager.getCurrentImage();
@@ -75,6 +75,8 @@ public class Selection implements PlugIn, Measurements {
 			toBoundingBox(imp); 
 		else if (arg.equals("rotate"))
 			rotate(imp); 
+		else if (arg.equals("translate"))
+			translate(imp); 
 		else if (arg.equals("enlarge"))
 			enlarge(imp); 
 		else if (arg.equals("rect"))
@@ -281,27 +283,47 @@ public class Selection implements PlugIn, Measurements {
 		Undo.setup(Undo.ROI, imp);
 		adjust = gd.getNextBoolean();
 		int sign = adjust ? -1 : 1;
-		FloatPolygon poly = roi.getInterpolatedPolygon(sign*interval, smooth);
-		int t = roi.getType();
-		int type = roi.isLine()?Roi.FREELINE:Roi.FREEROI;
-		if (t==Roi.POLYGON && interval>1.0)
-			type = Roi.POLYGON;
-		if ((t==Roi.RECTANGLE||t==Roi.OVAL||t==Roi.FREEROI) && interval>=8.0)
-			type = Roi.POLYGON;
-		if ((t==Roi.LINE||t==Roi.FREELINE) && interval>=8.0)
-			type = Roi.POLYLINE;
-		if (t==Roi.POLYLINE && interval>=8.0)
-			type = Roi.POLYLINE;
-		ImageCanvas ic = imp.getCanvas();
-		if (poly.npoints<=150 && ic!=null && ic.getMagnification()>=12.0)
-			type = roi.isLine()?Roi.POLYLINE:Roi.POLYGON;
-		Roi p = new PolygonRoi(poly,type);
+		Roi newRoi = null;
+		if (roi instanceof ShapeRoi && ((ShapeRoi)roi).getRois().length>1) {
+			// handle composite roi, thanks to Michael Ellis
+			Roi[] rois = ((ShapeRoi) roi).getRois();
+			ShapeRoi newShapeRoi = null;
+			for (Roi roi2 : rois) {
+				FloatPolygon fPoly = roi2.getInterpolatedPolygon(interval,smooth);
+				PolygonRoi polygon = new PolygonRoi(fPoly,PolygonRoi.POLYGON);
+				if (newShapeRoi==null) // First Roi is the outer boundary
+					newShapeRoi = new ShapeRoi(polygon);
+				else {
+					// Assume subsequent Rois are holes to be subtracted
+					ShapeRoi tempRoi = new ShapeRoi(polygon);
+					tempRoi.not(newShapeRoi);
+					newShapeRoi = tempRoi;
+				}
+			}
+			newRoi = newShapeRoi;
+		} else {
+			FloatPolygon poly = roi.getInterpolatedPolygon(sign*interval, smooth);
+			int t = roi.getType();
+			int type = roi.isLine()?Roi.FREELINE:Roi.FREEROI;
+			if (t==Roi.POLYGON && interval>1.0)
+				type = Roi.POLYGON;
+			if ((t==Roi.RECTANGLE||t==Roi.OVAL||t==Roi.FREEROI) && interval>=8.0)
+				type = Roi.POLYGON;
+			if ((t==Roi.LINE||t==Roi.FREELINE) && interval>=8.0)
+				type = Roi.POLYLINE;
+			if (t==Roi.POLYLINE && interval>=8.0)
+				type = Roi.POLYLINE;
+			ImageCanvas ic = imp.getCanvas();
+			if (poly.npoints<=150 && ic!=null && ic.getMagnification()>=12.0)
+				type = roi.isLine()?Roi.POLYLINE:Roi.POLYGON;
+			newRoi = new PolygonRoi(poly,type);
+		}
 		if (roi.getStroke()!=null)
-			p.setStrokeWidth(roi.getStrokeWidth());
-		p.setStrokeColor(roi.getStrokeColor());
-		p.setName(roi.getName());
-		transferProperties(roi, p);
-		imp.setRoi(p);
+			newRoi.setStrokeWidth(roi.getStrokeWidth());
+		newRoi.setStrokeColor(roi.getStrokeColor());
+		newRoi.setName(roi.getName());
+		transferProperties(roi, newRoi);
+		imp.setRoi(newRoi);
 	}
 	
 	private static void transferProperties(Roi roi1, Roi roi2) {
@@ -366,18 +388,54 @@ public class Selection implements PlugIn, Measurements {
 		return y+44.0;
 	}
 
+	/**
+		Smooth the specified line, preserving the end points.<br>
+		Author: Eugene Katrukha
+	*/
 	int[] smooth(int[] a, int n) {
-		FloatProcessor fp = new FloatProcessor(n, 1);
-		for (int i=0; i<n; i++)
-			fp.putPixelValue(i, 0, a[i]);
-		GaussianBlur gb = new GaussianBlur();
-		gb.blur1Direction(fp, 2.0, 0.01, true, 0);
-		IJ.showProgress(1.0);
-		for (int i=0; i<n; i++)
-			a[i] = (int)Math.round(fp.getPixelValue(i, 0));
+		float [] out = new float[n];
+		int i,j;
+		//no point in averaging 2 points
+		if(n<3)
+			return a;
+		//preserve end points
+		out[0]=	a[0];
+		out[n-1] = a[n-1];
+		//average middle points
+		for (i=1; i<(n-1); i++) {
+			out[i]=0.0f;			
+			for(j=(i-1);j<(i+2);j++){
+				out[i] += a[j];
+			}
+			out[i] /= 3.0f;
+		}
+		for (i=0; i<n; i++)
+			a[i] = (int)Math.round(out[i]);
 		return a;
 	}
-	
+
+	/**
+		Smooth the specified line, preserving the end points.<br>
+		Author: Eugene Katrukha
+	*/
+	float[] smooth(float[] a, int n) {
+		float [] out = new float[n];
+		int i,j;
+		//no point in averaging 2 points
+		if(n<3)
+			return a;
+		//preserve end points
+		out[0]=	a[0];
+		out[n-1] = a[n-1];
+		for (i=1; i<(n-1); i++) {
+			out[i]=0.0f;
+			for(j=(i-1);j<(i+2);j++)
+				out[i] += a[j];
+			out[i] /= 3.0f;
+		}
+		return out;
+	}
+		
 	float[] getCurvature(int[] x, int[] y, int n) {
 		float[] x2 = new float[n];
 		float[] y2 = new float[n];
@@ -438,17 +496,6 @@ public class Selection implements PlugIn, Measurements {
 		return p;
 	}
 	
-	float[] smooth(float[] a, int n) {
-		FloatProcessor fp = new FloatProcessor(n, 1);
-		for (int i=0; i<n; i++)
-			fp.setf(i, 0, a[i]);
-		GaussianBlur gb = new GaussianBlur();
-		gb.blur1Direction(fp, 2.0, 0.01, true, 0);
-		for (int i=0; i<n; i++)
-			a[i] = fp.getf(i, 0);
-		return a;
-	}
-	
 	float[] getCurvature(float[] x, float[] y, int n) {
 		float[] x2 = new float[n];
 		float[] y2 = new float[n];
@@ -505,7 +552,7 @@ public class Selection implements PlugIn, Measurements {
 		if (p!=null) {
 			Undo.setup(Undo.ROI, imp);
 			imp.deleteRoi();
-			Roi roi2 = new PolygonRoi(p, roi.POLYGON);
+			Roi roi2 = new PolygonRoi(p, Roi.POLYGON);
 			transferProperties(roi, roi2);
 			imp.setRoi(roi2);
 			IJ.showTime(imp, startTime, "Convex Hull ", 1);
@@ -540,7 +587,7 @@ public class Selection implements PlugIn, Measurements {
 		Prefs.useInvertingLut = false;
 		boolean selectAll = roi!=null && roi.getType()==Roi.RECTANGLE && roi.getBounds().width==imp.getWidth()
 			&& roi.getBounds().height==imp.getHeight() && imp.isThreshold();
-		boolean overlay = imp.getOverlay()!=null && imp.getProcessor().getMinThreshold()==ImageProcessor.NO_THRESHOLD;
+		boolean overlay = imp.getOverlay()!=null && !imp.isThreshold();
 		if (!overlay && (roi==null || selectAll)) {
 			createMaskFromThreshold(imp);
 			Prefs.useInvertingLut = useInvertingLut;
@@ -580,7 +627,7 @@ public class Selection implements PlugIn, Measurements {
 	
 	void createMaskFromThreshold(ImagePlus imp) {
 		ImageProcessor ip = imp.getProcessor();
-		if (ip.getMinThreshold()==ImageProcessor.NO_THRESHOLD) {
+		if (!ip.isThreshold()) {
 			IJ.error("Create Mask", "Area selection, overlay or thresholded image required");
 			return;
 		}
@@ -602,7 +649,7 @@ public class Selection implements PlugIn, Measurements {
 
 	void createSelectionFromMask(ImagePlus imp) {
 		ImageProcessor ip = imp.getProcessor();
-		if (ip.getMinThreshold()!=ImageProcessor.NO_THRESHOLD) {
+		if (ip.isThreshold()) {
 			IJ.runPlugIn("ij.plugin.filter.ThresholdToSelection", "");
 			return;
 		}
@@ -619,6 +666,8 @@ public class Selection implements PlugIn, Measurements {
 		if (Prefs.blackBackground)
 			threshold = (threshold==255)?0:255;
 		ip.setThreshold(threshold, threshold, ImageProcessor.NO_LUT_UPDATE);
+		if (!IJ.isMacro())
+			IJ.log("Create Selection: threshold not set; assumed to be "+threshold+"-"+threshold);
 		IJ.runPlugIn("ij.plugin.filter.ThresholdToSelection", "");
 	}
 	
@@ -764,13 +813,65 @@ public class Selection implements PlugIn, Measurements {
 			IJ.error("This command requires a selection.");
 			return false;
 		}
+		String name = roi.getName();
+		if (name==null) name = "";
+		int position = roi.getPosition();
+		int group = roi.getGroup();
+		Color color = roi.getStrokeColor();
+		Color fillColor = roi.getFillColor();
+		int width = (int)roi.getStrokeWidth();
 		RoiProperties rp = new RoiProperties(title, roi);
 		boolean ok = rp.showDialog();
+		if (Recorder.record) {
+			boolean groupChanged = false;
+			String name2 = roi.getName();
+			if (name2==null) name2 = "";
+			int position2 = roi.getPosition();
+			int group2 = roi.getGroup();
+			if (group2!=group) groupChanged=true;
+			Color color2 = roi.getStrokeColor();
+			Color fillColor2 = roi.getFillColor();
+			int width2 = (int)roi.getStrokeWidth();
+			if (Recorder.scriptMode()) {
+				Recorder.recordCall("roi = imp.getRoi();");
+				if (name2!=name)
+					Recorder.recordCall("roi.setName(\""+name2+"\");");
+				if (position2!=position)
+					Recorder.recordCall("roi.setPosition("+position2+");");
+				if (group2!=group)
+					Recorder.recordCall("roi.setGroup("+group2+");");
+				if (width2!=width)
+					Recorder.recordCall("roi.setStrokeWidth("+width2+");");
+				if (color2!=color && !groupChanged)
+					Recorder.recordCall("roi.setStrokeColor("+getColor(color2)+");");
+				if (fillColor2!=fillColor)
+					Recorder.recordCall("roi.setFillColor("+getColor(fillColor2)+");");
+				Recorder.recordCall("imp.draw();");
+			} else {
+				if (name2!=name)
+					Recorder.record("Roi.setName", name2);
+				if (groupChanged)
+					Recorder.record("Roi.setGroup", group2);
+				if (position2!=position)
+					Recorder.record("Roi.setPosition", position2);
+				if (color2!=color && !groupChanged)
+					Recorder.record("Roi.setStrokeColor", Colors.colorToString(color2));
+				if (fillColor2!=fillColor)
+					Recorder.record("Roi.setFillColor", Colors.colorToString(fillColor2));
+				if (width2!=width)
+					Recorder.record("Roi.setStrokeWidth", width2);
+			}
+			Recorder.disableCommandRecording();
+		}
 		if (IJ.debugMode)
 			IJ.log(roi.getDebugInfo());
 		return ok;
 	}
 	
+	private String getColor(Color color) {
+		return "new Color("+color.getRed()+","+color.getGreen()+","+color.getBlue()+")";
+	}
+	
 	private void makeBand(ImagePlus imp) {
 		Roi roi = imp.getRoi();
 		if (roi==null) {
@@ -823,16 +924,16 @@ public class Selection implements PlugIn, Measurements {
 		Prefs.blackBackground = saveBlackBackground;
 		ip = edm.getProcessor();
 		ip.setThreshold(0, n, ImageProcessor.NO_LUT_UPDATE);
-		int xx=-1, yy=-1;
-		for (int x=r.x; x<r.x+r.width; x++) {
-			for (int y=r.y; y<r.y+r.height; y++) {
+		int xx=-1, yy=-1;	//will become the start position for the Wand
+		for (int y=Math.max(r.y,0); y<Math.min(r.y+r.height, height); y++) {
+			for (int x=Math.max(r.x,0); x<Math.min(r.x+r.width, width); x++) {
 				if (ip.getPixel(x, y)<n) {
 					xx=x; yy=y;
 					break;
 				}
 			}
 			if (xx>=0||yy>=0)
-				break;
+			break;
 		}
 		int count = IJ.doWand(edm, xx, yy, 0, null);
 		if (count<=0) {
@@ -934,10 +1035,24 @@ public class Selection implements PlugIn, Measurements {
 			double pd = ((xp[i2min] - xp[imin]) * (yp[jmin] - yp[imin]) - (xp[jmin] - xp[imin]) * (yp[i2min] - yp[imin])) / Math.sqrt(Math.pow(xp[i2min] - xp[imin], 2) + Math.pow(yp[i2min] - yp [imin], 2)); // signed feret diameter
 			double pairAngle = Math.atan2( yp[i2min]- yp[imin], xp[i2min]- xp[imin]);
 			double minAngle = pairAngle + Math.PI/2;
-			double x1 = xp[imin] + Math.cos(pairAngle) * min_hmax + Math.cos(minAngle) * pd/2;
-			double y1 = yp[imin] + Math.sin(pairAngle) * min_hmax + Math.sin(minAngle) * pd/2;
-			double x2 = xp[imin] + Math.cos(pairAngle) * min_hmin + Math.cos(minAngle) * pd/2;
-			double y2 = yp[imin] + Math.sin(pairAngle) * min_hmin + Math.sin(minAngle) * pd/2;
+
+			// rectangle center and signed full height
+			double xm = xp[imin] + Math.cos(pairAngle) * (min_hmax + min_hmin)/2 + Math.cos(minAngle) * pd/2;
+			double ym = yp[imin] + Math.sin(pairAngle) * (min_hmax + min_hmin)/2 + Math.sin(minAngle) * pd/2;
+			double hm = min_hmax - min_hmin;
+			
+			if (minFD > Math.abs(hm)) { // ensure control axis is parallel to longer side
+				pairAngle = pairAngle - Math.PI/2;
+				minFD = Math.abs(hm);
+				hm = pd;
+				}
+			
+			if (pairAngle * hm > 0)	hm = -hm; // ensure first control point at the top
+	
+			double x1 = xm + Math.cos(pairAngle) * hm/2;
+			double y1 = ym + Math.sin(pairAngle) * hm/2;
+			double x2 = xm - Math.cos(pairAngle) * hm/2;
+			double y2 = ym - Math.sin(pairAngle) * hm/2;
 			Undo.setup(Undo.ROI, imp);
 			imp.deleteRoi();
 			Roi roi2 = new RotatedRectRoi(x1,  y1,  x2,  y2,  minFD);
@@ -947,6 +1062,39 @@ public class Selection implements PlugIn, Measurements {
 		}
 	}
 	
+	private void translate(ImagePlus imp) {
+		if (!imp.okToDeleteRoi())
+			return;
+		Roi roi = imp.getRoi();
+		String options = Macro.getOptions();
+		if (options!=null && options.contains("interpolation=")) {
+			IJ.run("Translate...", options); // run Image>Transform>Translate
+			return;
+		}
+		if (roi==null) {
+			noRoi("Translate");
+			return;
+		}
+		double dx = translateX;
+		double dy = translateY;
+		GenericDialog gd = new GenericDialog("Translate");
+		gd.addNumericField("X offset (pixels): ", dx, 0);
+		gd.addNumericField("Y offset (pixels): ", dy, 0);
+		gd.showDialog();
+		if (gd.wasCanceled())
+			return;
+		dx = gd.getNextNumber();
+		dy = gd.getNextNumber();
+		Rectangle2D r = roi.getFloatBounds();
+		roi.setLocation(r.getX()+dx, r.getY()+dy);
+		if (imp!=null)
+			imp.draw();
+		if (options==null) {
+			translateX = dx;
+			translateY = dy;
+		}
+	}
+	
 	void noRoi(String command) {
 		IJ.error(command, "This command requires a selection");
 	}
