@@ -173,13 +173,24 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 
 			
 	/** Constructs a ParticleAnalyzer.
-		@param options	a flag word created by Oring SHOW_RESULTS, EXCLUDE_EDGE_PARTICLES, etc.
-		@param measurements a flag word created by ORing constants defined in the Measurements interface
-		@param rt		a ResultsTable where the measurements will be stored
-		@param minSize	the smallest particle size in pixels
-		@param maxSize	the largest particle size in pixels
-		@param minCirc	minimum circularity
-		@param maxCirc	maximum circularity
+	 * @param options	a flag word created by Oring SHOW_RESULTS, EXCLUDE_EDGE_PARTICLES, etc.
+	 * @param measurements a flag word created by ORing constants defined in the Measurements interface
+	 * @param rt		a ResultsTable where the measurements will be stored
+	 * @param minSize	the smallest particle size in pixels
+	 * @param maxSize	the largest particle size in pixels
+	 * @param minCirc	minimum circularity
+	 * @param maxCirc	maximum circularity
+	 * <pre>
+	 * // JavaScript example
+	 * imp = IJ.openImage("https://imagej.net/images/blobs.gif");
+	 * IJ.setAutoThreshold(imp, "Default"); 
+	 * rt = new ResultsTable();
+	 * options = ParticleAnalyzer.SHOW_NONE;
+	 * measurements = Measurements.AREA + Measurements.PERIMETER;
+	 * pa = new ParticleAnalyzer(options, measurements, rt, 0, Double.MAX_VALUE, 0, 1);
+	 * pa.analyze(imp);
+	 * rt.show("Results");
+  	 * </pre>
 	*/
 	public ParticleAnalyzer(int options, int measurements, ResultsTable rt, double minSize, double maxSize, double minCirc, double maxCirc) {
 		this.options = options;
@@ -289,6 +300,8 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 				inSituShow = true;
 			if (mOptions.contains("record"))
 				recordStarts = true;
+			if (mOptions.contains("four"))
+				wandMode = Wand.FOUR_CONNECTED;
 			staticMinSize = 0.0; staticMaxSize = DEFAULT_MAX_SIZE;
 			staticMinCircularity=0.0; staticMaxCircularity=1.0;
 			staticShowChoice = NOTHING;
@@ -344,7 +357,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		labels[6]="Add to Manager"; states[6]=(options&ADD_TO_MANAGER)!=0;
 		labels[7]="Composite ROIs"; states[7]=(options&COMPOSITE_ROIS)!=0;
 		gd.addCheckboxGroup(4, 2, labels, states);
-		gd.addHelp(IJ.URL+"/docs/menus/analyze.html#ap");
+		gd.addHelp(IJ.URL2+"/docs/menus/analyze.html#ap");
 		gd.showDialog();
 		if (gd.wasCanceled())
 			return false;
@@ -410,6 +423,8 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		options |= SHOW_PROGRESS;
 		if ((options&DISPLAY_SUMMARY)!=0)
 			Analyzer.setMeasurements(Analyzer.getMeasurements()|AREA);
+		if (wandMode==Wand.FOUR_CONNECTED)
+			options |= INCLUDE_HOLES;
 		return true;
 	}
 	
@@ -441,8 +456,10 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		return true;
 	}
 
-	/** Performs particle analysis on the specified image. Returns
-		false if there is an error. */
+	/** Performs particle analysis on the current slice
+	 * of the specified image. Returns false if there
+	 * is an error.
+	*/
 	public boolean analyze(ImagePlus imp) {
 		return analyze(imp, imp.getProcessor());
 	}
@@ -456,7 +473,8 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 		resetCounter = (options&CLEAR_WORKSHEET)!=0;
 		showProgress = (options&SHOW_PROGRESS)!=0;
 		floodFill = (options&INCLUDE_HOLES)==0;
-		//recordStarts = (options&RECORD_STARTS)!=0;
+		if ((options&RECORD_STARTS)!=0)
+			recordStarts = true;
 		showOverlay = (options&OVERLAY)!=0;
 		addToManager = (options&ADD_TO_MANAGER)!=0;
 		if (staticRoiManager!=null) {
@@ -797,27 +815,30 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 			imageType = BYTE;
 		if (t1==ImageProcessor.NO_THRESHOLD) {
 			noThreshold = true;
-			ImageStatistics stats = imp.getStatistics();
+			ImageStatistics stats = imp.getRawStatistics();
 			if (imageType!=BYTE || (stats.histogram[0]+stats.histogram[255]!=stats.pixelCount)) {
 				IJ.error("Particle Analyzer",
-					"A thresholded image or 8-bit binary image is\n"
-					+"required. Threshold levels can be set using\n"
-					+"the Image->Adjust->Threshold tool.");
+					"A threshold has not been set using the\n"
+					+"Image->Adjust->Threshold tool or the \n"
+					+"setThreshold(min,max) macro function.");
 				canceled = true;
 				return false;
 			}
-			boolean threshold255 = invertedLut;
-			if (Prefs.blackBackground)
-				threshold255 = !threshold255;
-			if (threshold255) {
-				level1 = 255;
-				level2 = 255;
-				fillColor = 64;
-			} else {
+			level1 = 255;
+			level2 = 255;
+			fillColor = 64;
+			if (!Prefs.blackBackground && !invertedLut) {
 				level1 = 0;
 				level2 = 0;
 				fillColor = 192;
 			}
+			if (!IJ.isMacro()) {
+				boolean blackBackground = imageType==BYTE && Prefs.blackBackground && level1==255 && level1==level2;
+				if (!blackBackground) {
+					String suffix = !Prefs.blackBackground?" (\"Black background\" not set)":"";
+					IJ.log("ParticleAnalyzer: threshold not set; assumed to be "+(int)level1+"-"+(int)level2+suffix);
+				}
+			}
 		} else {
 			level1 = t1;
 			level2 = t2;
@@ -1007,6 +1028,7 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 				overlay = new Overlay();
 				overlay.drawLabels(true);
 				overlay.setLabelFont(new Font("SansSerif", Font.PLAIN, fontSize));
+				overlay.setDraggable(false);
 			}
 			Roi roi2 = (Roi)roi.clone();
 			roi2.setStrokeColor(Color.cyan);
@@ -1029,16 +1051,19 @@ public class ParticleAnalyzer implements PlugInFilter, Measurements {
 			overlay.add(roi2);
 		} else {
 			Rectangle r = roi.getBounds();
-			int nPoints = ((PolygonRoi)roi).getNCoordinates();
-			int[] xp = ((PolygonRoi)roi).getXCoordinates();
-			int[] yp = ((PolygonRoi)roi).getYCoordinates();
-			int x=r.x, y=r.y;
 			if (!inSituShow)
 				ip.setValue(0.0);
-			ip.moveTo(x+xp[0], y+yp[0]);
-			for (int i=1; i<nPoints; i++)
-				ip.lineTo(x+xp[i], y+yp[i]);
-			ip.lineTo(x+xp[0], y+yp[0]);
+			if (roi instanceof PolygonRoi) {
+				int nPoints = ((PolygonRoi)roi).getNCoordinates();
+				int[] xp = ((PolygonRoi)roi).getXCoordinates();
+				int[] yp = ((PolygonRoi)roi).getYCoordinates();
+				int x=r.x, y=r.y;
+				ip.moveTo(x+xp[0], y+yp[0]);
+				for (int i=1; i<nPoints; i++)
+					ip.lineTo(x+xp[i], y+yp[i]);
+				ip.lineTo(x+xp[0], y+yp[0]);
+			} else
+				roi.drawPixels(ip);
 			if (showChoice!=BARE_OUTLINES) {
 				String s = ResultsTable.d2s(count,0);
 				ip.moveTo(r.x+r.width/2-ip.getStringWidth(s)/2, r.y+r.height/2+fontSize/2);
