@@ -20,9 +20,7 @@ import ij.process.*;
 import ij.gui.*;
 import ij.io.*;
 import ij.plugin.filter.*;
-import ij.plugin.Colors;
-import ij.plugin.OverlayLabels;
-import ij.plugin.FolderOpener;
+import ij.plugin.*;
 import ij.util.*;
 import ij.macro.*;
 import ij.measure.*;
@@ -77,6 +75,11 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 	private boolean multiCropShow = true;
 	private boolean multiCropSave;
 	private int multiCropFormatIndex;
+	private static double angle = 45.0;
+	private static double xscale = 1.5;
+	private static double yscale = 1.5;
+	private static boolean scaleCentered = true;
+	
 
 	/** Opens the "ROI Manager" window, or activates it if it is already open.
 	 * @see #RoiManager(boolean)
@@ -100,7 +103,7 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		showWindow();
 	}
 
-	/** Constructs an ROIManager without displaying it. The boolean argument is ignored. */
+	/** Constructs an ROI Manager without displaying it. The boolean argument is ignored. */
 	public RoiManager(boolean b) {
 		super("ROI Manager");
 		list = new JList();
@@ -190,7 +193,10 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		addPopupItem("Labels...");
 		addPopupItem("List");
 		addPopupItem("Interpolate ROIs");
+		addPopupItem("Scale...");
+		addPopupItem("Rotate...");
 		addPopupItem("Translate...");
+		addPopupItem("ROI Manager Action");
 		addPopupItem("Help");
 		addPopupItem("Options...");
 		add(pm);
@@ -226,7 +232,9 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			open(null);
 		else if (command.equals("Save...")) {
 			Thread t1 = new Thread(new Runnable() {
-				public void run() {save();}
+				public void run() {
+					save(null);
+				}
 			});
 			t1.start();
 		} else if (command.equals("Fill"))
@@ -268,6 +276,10 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			listRois();
 		else if (command.equals("Interpolate ROIs"))
 			interpolateRois();
+		else if (command.equals("Scale..."))
+			scale();
+		else if (command.equals("Rotate..."))
+			rotate();
 		else if (command.equals("Translate..."))
 			translate();
 		else if (command.equals("Help"))
@@ -276,6 +288,8 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			options();
 		else if (command.equals("\"Show All\" Color..."))
 			setShowAllColor();
+		else if (command.equals("ROI Manager Action"))
+  			IJ.run(command);
 		allowRecording = false;
 	}
 
@@ -328,7 +342,8 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 	}
 
 	private boolean okToSet() {
-		return !(IJ.isMacOSX()&&IJ.isJava18());
+		//return !(IJ.isMacOSX()&&IJ.isJava18());
+		return false;
 	}
 
 	void add(boolean shiftKeyDown, boolean altKeyDown) {
@@ -515,9 +530,9 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			isStandard = true;
 		else if (len>=17 && name.charAt(5)=='-' && name.charAt(11)=='-' )
 			isStandard = true;
-		else if (len>=9 && name.charAt(4)=='-')
+		else if (len>=9 && name.charAt(4)=='-' && Character.isDigit(name.charAt(5)))
 			isStandard = true;
-		else if (len>=11 && name.charAt(5)=='-')
+		else if (len>=11 && name.charAt(5)=='-' && Character.isDigit(name.charAt(6)))
 			isStandard = true;
 		return isStandard;
 	}
@@ -671,10 +686,11 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		rois.set(index, roi);
 		listModel.setElementAt(name2, index);
 		list.setSelectedIndex(index);
-		if (Prefs.useNamesAsLabels && labelsCheckbox.getState()) {
-			ImagePlus imp = WindowManager.getCurrentImage();
+		ImagePlus imp = WindowManager.getCurrentImage();
+		if (imp!=null && imp.getRoi()!=null)
+			imp.setRoi(roi);
+		if (Prefs.useNamesAsLabels && labelsCheckbox.getState())
 			if (imp!=null) imp.draw();
-		}
 		if (record())
 			Recorder.record("roiManager", "Rename", name2);
 		return true;
@@ -702,31 +718,36 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		Roi roi = (Roi)rois.get(index);
 		if (imp==null || roi==null)
 			return false;
+			//IJ.log("restore: "+roi.getPosition()+"  "+roi.getZPosition()+"  "+imp.getNSlices()+"  "+imp.getStackSize());
 		if (setSlice) {
-			if (imp.lock()) {
-				boolean hyperstack = imp.isHyperStack();
-				if (hyperstack && roi.hasHyperStackPosition())
-					imp.setPosition(roi.getCPosition(), roi.getZPosition(), roi.getTPosition());
-				else {
-					String label = (String)listModel.getElementAt(index);
-					int n = getSliceNumber(roi, label);
-					if (n>=1 && n<=imp.getStackSize()) {
-						if (hyperstack) {
-							if (imp.getNSlices()>1 && n<=imp.getNSlices())
-								imp.setPosition(imp.getC(),n,imp.getT());
-							else if (imp.getNFrames()>1 && n<=imp.getNFrames())
-								imp.setPosition(imp.getC(),imp.getZ(),n);
-							else
-								imp.setPosition(n);
-						} else
-							imp.setSlice(n);
-					} else if (roi.getZPosition()>0 && imp.getNSlices()==imp.getStackSize())
-						imp.setSlice(roi.getZPosition());
+			boolean hyperstack = imp.isHyperStack();
+			int position = roi.getPosition();
+			if (hyperstack && roi.hasHyperStackPosition())
+				imp.setPosition(roi.getCPosition(), roi.getZPosition(), roi.getTPosition());
+			else if (hyperstack && imp.getNSlices()==1)
+				imp.setPosition(imp.getChannel(), 1, position);
+			else if (hyperstack)
+				imp.setPosition(imp.getChannel(), position, imp.getChannel());
+			else if (roi.getZPosition()>0 && imp.getNSlices()==imp.getStackSize())
+				imp.setSlice(roi.getZPosition());
+			else if (position>0 && position<=imp.getStackSize())
+				imp.setSlice(position);
+			else {
+				String label = (String)listModel.getElementAt(index);
+				int n = getSliceNumber(roi, label);
+				if (n>=1 && n<=imp.getStackSize()) {
+					if (hyperstack) {
+						if (imp.getNSlices()>1 && n<=imp.getNSlices())
+							imp.setPosition(imp.getC(),n,imp.getT());
+						else if (imp.getNFrames()>1 && n<=imp.getNFrames())
+							imp.setPosition(imp.getC(),imp.getZ(),n);
+						else
+							imp.setPosition(n);
+					} else
+						imp.setSlice(n);
 				}
-				imp.unlock();
-			} else
-				return false;
-		}
+			}
+		}	
 		if (showAllCheckbox.getState() && !restoreCentered && !noUpdateMode) {
 			roi.setImage(null);
 			imp.setRoi(roi);
@@ -786,7 +807,10 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		return slice;
 	}
 
-	void open(String path) {
+	/** Opens a single .roi file or a ZIP-compressed set of ROIs.
+	 *	Returns 'true' if the operation was succesful.
+	*/
+	public boolean open(String path) {
 		Macro.setOptions(null);
 		String name = null;
 		if (path==null || path.equals("")) {
@@ -794,17 +818,22 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			String directory = od.getDirectory();
 			name = od.getFileName();
 			if (name==null)
-				return;
+				return false;
 			path = directory + name;
 		}
-		if (Recorder.record && !Recorder.scriptMode())
-			Recorder.record("roiManager", "Open", path);
+		if (Recorder.record && !IJ.macroRunning()) {
+			if (Recorder.scriptMode())
+				Recorder.recordCall("rm.open(\""+path+"\");");
+			else
+				Recorder.record("roiManager", "Open", path);
+		}
+		boolean ok = false;
 		if (path.endsWith(".zip")) {
 			boolean wasRecording = Recorder.record;
 			Recorder.record = false;
-			openZip(path);
+			ok = openZip(path);
 			Recorder.record = wasRecording;
-			return;
+			return ok;
 		}
 		Opener o = new Opener();
 		if (name==null) name = o.getName(path);
@@ -817,13 +846,17 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			listModel.addElement(name);
 			rois.add(roi);
 			errorMessage = null;
-		} else
-			errorMessage = "Unable to 	open ROI at "+path;
+			ok = true;
+		} else {
+			errorMessage = "Unable to open ROI at "+path;
+			ok = false;
+		}
 		updateShowAll();
+		return ok;
 	}
 
 	// Modified on 2005/11/15 by Ulrik Stervbo to only read .roi files and to not empty the current list
-	void openZip(String path) {
+	boolean openZip(String path) {
 		ZipInputStream in = null;
 		ByteArrayOutputStream out = null;
 		int nRois = 0;
@@ -867,16 +900,24 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			error(errorMessage);
 		}
 		updateShowAll();
+		return errorMessage==null;
 	}
 
-	boolean save() {
+	/** If one ROI is selected, it is saved as a .roi
+	 * file, if multiple (or no) ROIs are selected,
+	 * they are saved as a .zip ROI set. Returns 
+	 * 'true' if the save operation was succesful.
+	 * @see #setSelectedIndexes
+	*/
+	public boolean save(String path) {
 		if (getCount()==0)
 			return error("The selection list is empty.");
 		int[] indexes = getIndexes();
 		if (indexes.length>1)
-			return saveMultiple(indexes, null);
+			return saveMultiple(indexes, path);
 		else
-			return saveOne(indexes, null);
+			return saveOne(indexes, path);
+
 	}
 
 	boolean saveOne(int[] indexes, String path) {
@@ -906,14 +947,18 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			errorMessage = e.getMessage();
 			IJ.error("ROI Manager", errorMessage);
 		}
-		if (Recorder.record && !IJ.isMacro())
-			Recorder.record("roiManager", "Save", path);
+		if (Recorder.record && !IJ.isMacro()) {
+			if (Recorder.scriptMode())
+				Recorder.recordCall("rm.save(\""+path+"\");");
+			else
+				Recorder.record("roiManager", "Save", path);
+		}
 		return true;
 	}
 
 	boolean saveMultiple(int[] indexes, String path) {
 		Macro.setOptions(null);
-		if (path==null) {
+		if (path==null || path.equals("")) {
 			SaveDialog sd = new SaveDialog("Save ROIs...", "RoiSet", ".zip");
 			String name = sd.getFileName();
 			if (name == null)
@@ -936,12 +981,18 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			RoiEncoder re = new RoiEncoder(out);
 			for (int i=0; i<indexes.length; i++) {
 				IJ.showProgress(i, indexes.length);
-				String label = getUniqueName(names, indexes[i]);
+				String label = (String)listModel.getElementAt(i);
 				Roi roi = (Roi)rois.get(indexes[i]);
 				if (IJ.debugMode) IJ.log("saveMultiple: "+i+"  "+label+"  "+roi);
 				if (roi==null) continue;
 				if (!label.endsWith(".roi")) label += ".roi";
-				zos.putNextEntry(new ZipEntry(label));
+				try {
+					zos.putNextEntry(new ZipEntry(label));
+				} catch(ZipException e) {
+					label = getUniqueName(names, indexes[i]);
+					if (!label.endsWith(".roi")) label += ".roi";
+					zos.putNextEntry(new ZipEntry(label));
+				}
 				re.write(roi);
 				out.flush();
 			}
@@ -957,8 +1008,12 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		double time = (System.currentTimeMillis()-t0)/1000.0;
 		IJ.showProgress(1.0);
 		IJ.showStatus(IJ.d2s(time,3)+" seconds, "+indexes.length+" ROIs, "+path);
-		if (Recorder.record && !IJ.isMacro())
-			Recorder.record("roiManager", "Save", path);
+		if (Recorder.record && !IJ.isMacro()) {
+			if (Recorder.scriptMode())
+				Recorder.recordCall("rm.save(\""+path+"\");");
+			else
+				Recorder.record("roiManager", "Save", path);
+		}
 		return true;
 	}
 
@@ -1165,7 +1220,7 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		return true;
 	}
 
-	private static ResultsTable multiMeasure(ImagePlus imp, Roi[] rois, boolean appendResults) {
+	public static ResultsTable multiMeasure(ImagePlus imp, Roi[] rois, boolean appendResults) {
 		int nSlices = imp.getStackSize();
 		Analyzer aSys = new Analyzer(imp); // System Analyzer
 		ResultsTable rtSys = Analyzer.getResultsTable();
@@ -1388,7 +1443,7 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		if (imp!=null) imp.draw();
 	}
 
-	/** Sets the position for the selected ROIs. */ 
+	/** Sets the position of the selected ROIs. */ 
 	public void setPosition(int position) {
 		int[] indexes = getIndexes();
 		for (int i: indexes) {
@@ -1399,6 +1454,17 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		if (imp!=null) imp.draw();
 	}
 
+	/** Sets the c,z,t position of the selected ROIs. */ 
+	public void setPosition(int c, int z, int t) {
+		int[] indexes = getIndexes();
+		for (int i: indexes) {
+			Roi roi = getRoi(i);
+			roi.setPosition(c, z, t);
+		}
+		ImagePlus imp = WindowManager.getCurrentImage();
+		if (imp!=null) imp.draw();
+	}
+
 	/** Obsolete; replaced by RoiManager.setGroup() macro function. */
 	public static void setGroup(String group) {
 		RoiManager rm = getInstance();
@@ -1474,7 +1540,7 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 				fillColor =	 rpRoi.getFillColor();
 				rpName = rpRoi.getName();
 			}
-			if (rpRoi.getStrokeColor()==null)
+			if (rpRoi.getStrokeColor()==null && rpRoi.getFillColor()==null)
 				rpRoi.setStrokeColor(Roi.getColor());
 			rpRoi = (Roi) rpRoi.clone();
 			if (n>1)
@@ -1504,7 +1570,7 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			}
 		}
 		ImagePlus imp = WindowManager.getCurrentImage();
-		if (n==getCount() && n>1 && !IJ.isMacro()) {
+		if (n==getCount() && n>1 && !IJ.isMacro() && imp!=null && imp.getWindow()!=null) {
 			GenericDialog gd = new GenericDialog("ROI Manager");
 			gd.addMessage("Apply changes to all "+n+" selections?");
 			gd.showDialog();
@@ -1513,8 +1579,10 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		for (int i=0; i<n; i++) {
 			Roi roi = (Roi)rois.get(indexes[i]);
 			if (roi==null) continue;
-			if (color!=null) roi.setStrokeColor(color);
-			if (lineWidth>=0) roi.setStrokeWidth(lineWidth);
+			if (color!=null)
+				roi.setStrokeColor(color);
+			if (lineWidth>=0)
+				roi.setStrokeWidth(lineWidth);
 			roi.setFillColor(fillColor);
 			if (group>=0)
 				roi.setGroup(group);
@@ -1524,7 +1592,7 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 				else
 					roi.setPosition(rpRoi.getPosition());
 			}
-			if (roi instanceof TextRoi) {
+			if ((roi instanceof TextRoi) && showDialog) {
 				roi.setImage(imp);
 				if (font!=null)
 					((TextRoi)roi).setCurrentFont(font);
@@ -1644,7 +1712,7 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			 roi = Roi.convertLineToArea(roi);
 			if (s1==null) {
 				if (roi instanceof ShapeRoi)
-					s1 = (ShapeRoi)roi;
+					s1 = (ShapeRoi)roi.clone();
 				else
 					s1 = new ShapeRoi(roi);
 				if (s1==null) return;
@@ -1726,34 +1794,13 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 	void xor() {
 		ImagePlus imp = getImage();
 		if (imp==null) return;
-		int[] indexes = getSelectedIndexes();
-		if (indexes.length==1) {
+		Roi[] rois = getSelectedRoisAsArray();
+		if (rois.length==1) {
 			error("More than one item must be selected, or none");
 			return;
 		}
-		if (indexes.length==0)
-			indexes = getAllIndexes();
-		ShapeRoi s1=null, s2=null;
-		for (int i=0; i<indexes.length; i++) {
-			Roi roi = (Roi)rois.get(indexes[i]);
-			if (roi==null)
-				continue;
-			if (s1==null) {
-				if (roi instanceof ShapeRoi)
-					s1 = (ShapeRoi)roi.clone();
-				else
-					s1 = new ShapeRoi(roi);
-				if (s1==null) return;
-			} else {
-				if (roi instanceof ShapeRoi)
-					s2 = (ShapeRoi)roi.clone();
-				else
-					s2 = new ShapeRoi(roi);
-				if (s2==null) continue;
-				s1.xor(s2);
-			}
-		}
-		if (s1!=null) imp.setRoi(s1.trySimplify());
+		Roi roi2 = Roi.xor(getSelectedRoisAsArray());
+		if (roi2!=null) imp.setRoi(roi2);
 		if (record()) Recorder.record("roiManager", "XOR");
 	}
 
@@ -1861,7 +1908,7 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 	}
 
 	private void help() {
-		String macro = "run('URL...', 'url="+IJ.URL+"/docs/menus/analyze.html#manager');";
+		String macro = "run('URL...', 'url="+IJ.URL2+"/docs/menus/analyze.html#manager');";
 		new MacroRunner(macro);
 	}
 
@@ -1910,9 +1957,15 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 				imp.draw();
 		}
 		if (record()) {
-			Recorder.record("roiManager", "Associate", Prefs.showAllSliceOnly?"true":"false");
-			Recorder.record("roiManager", "Centered", restoreCentered?"true":"false");
-			Recorder.record("roiManager", "UseNames", Prefs.useNamesAsLabels?"true":"false");
+			if (Recorder.scriptMode()) {
+				Recorder.recordString("Prefs.showAllSliceOnly = "+Prefs.showAllSliceOnly+";\n");
+				Recorder.recordString("RoiManager.restoreCentered("+restoreCentered+");\n");
+				Recorder.recordString("Prefs.useNamesAsLabels = "+Prefs.useNamesAsLabels+";\n");
+			} else {
+				Recorder.recordString("RoiManager.associateROIsWithSlices("+Prefs.showAllSliceOnly+");\n");
+				Recorder.recordString("RoiManager.restoreCentered("+restoreCentered+");\n");
+				Recorder.recordString("RoiManager.useNamesAsLabels("+Prefs.useNamesAsLabels+");\n");
+			}
 		}
 	}
 
@@ -1958,7 +2011,8 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			imageID = imp.getID();
 		if (mode==LABELS || mode==NO_LABELS)
 			showAll = true;
-		if (showAll) imp.deleteRoi();
+		if (showAll)
+			imp.deleteRoi();
 		if (mode==SHOW_NONE) {
 			removeOverlay(imp);
 			imageID = 0;
@@ -2020,8 +2074,9 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			ignoreInterrupts = false;
 	}
 
-	/** Returns a reference to the ROI Manager and opens
-		 the "ROI Manager" window if it is not already open. */
+	/** Returns a reference to the ROI Manager if it
+	 * open or opens it if it is not already open.
+	*/
 	public static RoiManager getRoiManager() {
 		if (instance!=null)
 			return (RoiManager)instance;
@@ -2135,16 +2190,29 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 	}
 
 	/** Returns the name of the ROI with the specified index,
-		or null if the index is out of range.
-		See also: RoiManager.getName() macro function.
+	 * or null if the index is out of range.
+	 * See also: RoiManager.getName() macro function.
 	*/
 	public String getName(int index) {
 		if (index>=0 && index<getCount())
-			return	(String) listModel.getElementAt(index);
+			return	(String)listModel.getElementAt(index);
 		else
 			return null;
 	}
 
+	/** Returns the index of the first ROI with the
+	 * specified name, or -1 if no ROI has that name.
+	 * See also: RoiManager.getIndex() macro function.
+	*/
+	public int getIndex(String name) {
+		Roi[] rois = getRoisAsArray();
+		for (int i=0; i<rois.length; i++) {
+			if (name.equals(rois[i].getName()))
+				return i;
+		}
+		return -1;
+	}
+
 	/** Returns the name of the ROI with the specified index.
 		Can be called from a macro using
 		<pre>call("ij.plugin.frame.RoiManager.getName", index)</pre>
@@ -2282,19 +2350,21 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		cmd = cmd.toLowerCase();
 		macro = true;
 		if (cmd.equals("open")) {
-			open(name);
+			boolean ok = open(Opener.makeFullPath(name));
 			macro = false;
-			return true;
+			return ok;
 		} else if (cmd.equals("save")) {
+			boolean ok = false;
 			if (name!=null && name.endsWith(".roi"))
-				saveOne(getIndexes(), name);
+				ok = saveOne(getIndexes(), name);
 			else
-				save(name, false);
+				ok = save(name, false);
+			return ok;
 		} else if (cmd.equals("save selected")) {
 			if (name!=null && name.endsWith(".roi"))
-				saveOne(getIndexes(), name);
+				return saveOne(getIndexes(), name);
 			else
-				save(name, true);
+				return save(name, true);
 		} else if (cmd.equals("rename")) {
 			rename(name);
 			macro = false;
@@ -2339,12 +2409,97 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 	public void reset() {
 		if (IJ.isMacOSX() && IJ.isMacro())
 			ignoreInterrupts = true;
-		listModel.removeAllElements();
+		if (listModel!=null)
+			listModel.removeAllElements();
 		overlayTemplate = null;
 		rois.clear();
 		updateShowAll();
 	}
 
+	private void scale() {
+		GenericDialog gd = new GenericDialog("Scale");
+		gd.addNumericField("X scale factor:", xscale, 2, 4, "");
+		gd.addNumericField("Y scale factor:", yscale, 2, 4, "");
+		gd.addCheckbox("Centered", scaleCentered);
+		gd.showDialog();
+		if (gd.wasCanceled())
+			return;
+		xscale = gd.getNextNumber();
+		yscale = gd.getNextNumber();
+		scaleCentered = gd.getNextBoolean();
+		scale(xscale, yscale, scaleCentered);
+		if (record()) {
+			if (Recorder.scriptMode())
+				Recorder.recordCall("rm.scale("+xscale+", "+yscale+", "+scaleCentered+");");
+			else
+				Recorder.recordString("RoiManager.scale("+xscale+", "+yscale+", "+scaleCentered+");\n");
+		}
+	}
+
+	public void scale(double xscale, double yscale, boolean centered) {
+		int[] indexes = getIndexes();
+		for (int i=0; i<indexes.length; i++) {
+			Roi roi = (Roi)rois.get(indexes[i]);
+			Roi roi2 = RoiScaler.scale(roi, xscale, yscale, centered);
+			rois.set(indexes[i], roi2);
+		}
+		updateShowAll();
+	}
+
+	private void rotate() {
+		double xcenter = Double.NaN;
+		double ycenter = Double.NaN;
+		GenericDialog gd = new GenericDialog("Rotate");
+		gd.addNumericField("Angle:", angle, 2, 5, "degrees");
+		gd.addCheckbox("Rotate around image center", false);
+		gd.showDialog();
+		if (gd.wasCanceled())
+			return;
+		angle = gd.getNextNumber();
+		if (angle==0.0)
+			return;
+		boolean rotateAroundImageCenter = gd.getNextBoolean();
+		ImagePlus imp = WindowManager.getCurrentImage();
+		if (imp!=null && rotateAroundImageCenter) {
+			xcenter = imp.getWidth()/2.0;
+			ycenter = imp.getHeight()/2.0;
+		}
+		rotate(angle, xcenter, ycenter);
+		if (record()) {
+			if (Recorder.scriptMode()) {
+				if (Double.isNaN(xcenter))
+					Recorder.recordCall("rm.rotate("+angle+");");
+				else
+					Recorder.recordCall("rm.rotate("+angle+", "+xcenter+", "+ycenter+");");
+			} else {
+				if (Double.isNaN(xcenter))
+					Recorder.recordString("RoiManager.rotate("+angle+");\n");
+				else
+					Recorder.recordString("RoiManager.rotate("+angle+", "+xcenter+", "+ycenter+");\n");
+			}
+		}
+	}
+	
+	public void rotate(double angle) {
+		rotate(angle, Double.NaN, Double.NaN);
+	}
+
+	public void rotate(double angle, double xcenter, double ycenter) {
+		boolean useRoiCenter = Double.isNaN(xcenter);
+		int[] indexes = getIndexes();
+		for (int i=0; i<indexes.length; i++) {
+			Roi roi = (Roi)rois.get(indexes[i]);
+			if (useRoiCenter) {
+				FloatPolygon center = roi.getRotationCenter();
+				xcenter = center.xpoints[0];
+				ycenter = center.ypoints[0];
+			}
+			Roi roi2 = RoiRotator.rotate(roi, angle, xcenter, ycenter);
+			rois.set(indexes[i], roi2);
+		}
+		updateShowAll();
+	}
+
 	private void translate() {
 		GenericDialog gd = new GenericDialog("Translate");
 		gd.addNumericField("X offset (pixels): ", translateX, 0);
@@ -2359,18 +2514,15 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			if (Recorder.scriptMode())
 				Recorder.recordCall("rm.translate("+translateX+", "+translateY+");");
 			else
-				Recorder.record("roiManager", "translate", (int)translateX, (int)translateY);
+				Recorder.record("RoiManager.translate", (int)translateX, (int)translateY);
 		}
 	}
 
 	/** Moves the selected ROIs or all the ROIs if none are selected. */
 	public void translate(double dx, double dy) {
 		Roi[] rois = getSelectedRoisAsArray();
-		for (int i=0; i<rois.length; i++) {
-			Roi roi = rois[i];
-			Rectangle2D r = roi.getFloatBounds();
-			roi.setLocation(r.getX()+dx, r.getY()+dy);
-		}
+		for (int i=0; i<rois.length; i++)
+			rois[i].translate(dx,dy);
 		ImagePlus imp = WindowManager.getCurrentImage();
 		if (imp!=null)
 			imp.draw();
@@ -2424,7 +2576,8 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		int n = getCount();
 		if (index>=n) return;
 		boolean mm = list.getSelectionMode() == ListSelectionModel.MULTIPLE_INTERVAL_SELECTION;
-		if (mm) list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
+		if (mm)
+			list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
 		int delay = 1;
 		long start = System.currentTimeMillis();
 		while (true) {
@@ -2437,7 +2590,8 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			imp = WindowManager.getCurrentImage();
 		if (imp!=null)
 			restore(imp, index, true);
-		if (mm) list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
+		if (mm)
+			list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
 	}
 
 	public void selectAndMakeVisible(ImagePlus imp, int index) {
@@ -2478,6 +2632,23 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		setSelectedIndexes(selected);
 	}
 
+	/** Selects all ROIs at the specified position. */
+	public void selectPosition(int c, int z, int t) {
+		ArrayList<Integer>listSelected = new ArrayList<Integer>();
+		for (int i=0; i<getCount(); i++) {
+			Roi roi = getRoi(i);
+			int c2 = roi.getCPosition();
+			int z2 = roi.getZPosition();
+			int t2 = roi.getTPosition();
+			if ((c!=0&&c==c2) || (z!=0&&z==z2) || (t!=0&&t==t2))
+				listSelected.add(i);
+		}
+		int[] selected = new int[listSelected.size()];
+		for (int j=0; j<listSelected.size(); j++)
+			selected[j] = listSelected.get(j);
+		setSelectedIndexes(selected);
+	}
+
 	/** Obsolete; replaced by RoiManager.selectGroup() macro function. */
 	public static void selectGroup(String group) {
 		RoiManager rm = getInstance();
@@ -2549,10 +2720,6 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			overlay = newOverlay();
 		for (int i=0; i<n; i++) {
 			Roi roi = (Roi)rois[i].clone();
-			if (!Prefs.showAllSliceOnly && !IJ.isMacro())
-				roi.setPosition(0);
-			//if (roi.getStrokeWidth()==1)
-			//	roi.setStrokeWidth(0);
 			overlay.add(roi);
 		}
 		if (overlayTemplate!=null)
@@ -2586,9 +2753,6 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 		}
 	}
 
-	/** Selects multiple ROIs, where 'indexes' is an array of integers,
-		each greater than or equal to 0 and less than the value returned by getCount().
-	*/
 	/** Selects multiple ROIs, where 'indexes' is an array of integers, each
 	* greater than or equal to 0 and less than the value returned by getCount().
 	* @see #getSelectedIndexes
@@ -2661,7 +2825,8 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 			return;
 		ImageCanvas ic = imp.getCanvas();
 		if (ic==null) {
-			imp.setOverlay(overlay);
+			if (imp.getOverlay()==null)
+				imp.setOverlay(overlay);
 			return;
 		}
 		ic.setShowAllList(overlay);
@@ -2679,9 +2844,21 @@ public class RoiManager extends PlugInFrame implements ActionListener, ItemListe
 	public void allowRecording(boolean allow) {
 		this.allowRecording = allow;
 	}
+	
+	public static void restoreCentered(boolean b) {
+		restoreCentered = b;
+	}
+	
+	/* handle double clicking on a ROI. */
+	public void mouseClicked (MouseEvent e) {
+		if (e.getClickCount() == 2 && !e.isConsumed()) {
+			e.consume();
+			if (ij.plugin.MacroInstaller.isMacroCommand("ROI Manager Action"))
+				IJ.run("ROI Manager Action");
+		}
+	}
 
 	public void mouseReleased (MouseEvent e) {}
-	public void mouseClicked (MouseEvent e) {}
 	public void mouseEntered (MouseEvent e) {}
 	public void mouseExited (MouseEvent e) {}
 
