@@ -73,6 +73,7 @@ public class ColorProcessor extends ImageProcessor {
 			rgbRaster = Raster.createWritableRaster(rgbSampleModel, dataBuffer, null);
 		}
 		if (image==null) {
+			if (cm==null) createColorModel();
 			image = new BufferedImage(cm, rgbRaster, false, null);
 		}
 		return image;
@@ -87,7 +88,7 @@ public class ColorProcessor extends ImageProcessor {
 
 	public void setColorModel(ColorModel cm) {
 		if (cm!=null && (cm instanceof IndexColorModel))
-			throw new IllegalArgumentException("DirectColorModel required");
+			throw new IllegalArgumentException("RGB images do not support IndexColorModels");
 		this.cm = cm;
 		rgbSampleModel = null;
 		rgbRaster = null;
@@ -114,6 +115,11 @@ public class ColorProcessor extends ImageProcessor {
 		drawingColor = color;
 		fillValueSet = true;
 	}
+	
+	/** Sets the background fill/draw color. */
+	public void setBackgroundColor(Color color) {
+		setBackgroundValue(color.getRGB());
+	}
 
 	/** Sets the fill/draw color, where <code>color</code> is an RGB int. */
 	public void setColor(int color) {
@@ -182,7 +188,6 @@ public class ColorProcessor extends ImageProcessor {
 			applyTable(lut, channels);
 	}
 	
-
 	public void snapshot() {
 		snapshotWidth = width;
 		snapshotHeight = height;
